/***************************************************************************
 *  assembler.h
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Modo di Indirizzamento:
 *      0       noarg - Nessun operando
 *      1       imm - Immediato
 *      2       data - Data
 *      3       bit - Bit
 *      4       imm16 - 16 bit (mov DPTR,#...)
 *      5       reladdr - <jmp> <rel>
 *      6       fixedarg - JMP @A+DPTR/RET/RETI (nessun argomento)
 *      7       absaddr - LCALL/LJMP
 *      0x80..0x87 paddr80 .. paddr87 - (AJMP/ACALL) 0..7 -> bit 10..8, byte seguente -> byte 7..0
 *
 *	char *Mnemonic;
 *	argType Arg1,Arg2;
 *	argParseType p1,p2,p3;
 *	unsigned char Len;
 *	unsigned char Cycles;
 */
OpCode i8052::IstSet [] = {
  /* 00 */  OpCode ("NOP",OpCode::noarg,OpCode::noarg,OpCode::noParg,OpCode::noParg,OpCode::noParg,1,1),
  /* 01 */  OpCode ("AJMP %s",OpCode::paddr80,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 02 */  OpCode ("LJMP %s",OpCode::absaddr,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,3,1),
  /* 03 */  OpCode ("RR A",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::noParg,OpCode::noParg,1,1),
  /* 04 */  OpCode ("INC A",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::noParg,OpCode::noParg,1,1),
  /* 05 */  OpCode ("INC %s",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,1),
  /* 06 */  OpCode ("INC @R0",OpCode::noarg,OpCode::noarg,OpCode::indReg,OpCode::noParg,OpCode::noParg,1,1),
  /* 07 */  OpCode ("INC @R1",OpCode::noarg,OpCode::noarg,OpCode::indReg,OpCode::noParg,OpCode::noParg,1,1),
  /* 08 */  OpCode ("INC R0",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 09 */  OpCode ("INC R1",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 0A */  OpCode ("INC R2",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 0B */  OpCode ("INC R3",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 0C */  OpCode ("INC R4",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 0D */  OpCode ("INC R5",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 0E */  OpCode ("INC R6",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 0F */  OpCode ("INC R7",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 10 */  OpCode ("JBC %s,%s",OpCode::bit,OpCode::reladdr,OpCode::bitDir,OpCode::dir,OpCode::noParg,3,1),
  /* 11 */  OpCode ("ACALL %s",OpCode::paddr80,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 12 */  OpCode ("LCALL %s",OpCode::absaddr,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,3,1),
  /* 13 */  OpCode ("RRC A",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::noParg,OpCode::noParg,1,1),
  /* 14 */  OpCode ("DEC A",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::noParg,OpCode::noParg,1,1),
  /* 15 */  OpCode ("DEC %s",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,1),
  /* 16 */  OpCode ("DEC @R0",OpCode::noarg,OpCode::noarg,OpCode::indReg,OpCode::noParg,OpCode::noParg,1,1),
  /* 17 */  OpCode ("DEC @R1",OpCode::noarg,OpCode::noarg,OpCode::indReg,OpCode::noParg,OpCode::noParg,1,1),
  /* 18 */  OpCode ("DEC R0",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 19 */  OpCode ("DEC R1",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 1A */  OpCode ("DEC R2",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 1B */  OpCode ("DEC R3",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 1C */  OpCode ("DEC R4",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 1D */  OpCode ("DEC R5",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 1E */  OpCode ("DEC R6",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 1F */  OpCode ("DEC R7",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::noParg,OpCode::noParg,1,1),
  /* 20 */  OpCode ("JB %s,%s",OpCode::bit,OpCode::reladdr,OpCode::bitDir,OpCode::dir,OpCode::noParg,3,1),
  /* 21 */  OpCode ("AJMP %s",OpCode::paddr81,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 22 */  OpCode ("RET",OpCode::fixedarg,OpCode::noarg,OpCode::noParg,OpCode::noParg,OpCode::noParg,1,2),
  /* 23 */  OpCode ("RL A",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::noParg,OpCode::noParg,1,1),
  /* 24 */  OpCode ("ADD A,#%s",OpCode::imm,OpCode::noarg,OpCode::aReg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 25 */  OpCode ("ADD A,%s",OpCode::data,OpCode::noarg,OpCode::aReg,OpCode::dir,OpCode::noParg,2,1),
  /* 26 */  OpCode ("ADD A,@R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 27 */  OpCode ("ADD A,@R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 28 */  OpCode ("ADD A,R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 29 */  OpCode ("ADD A,R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 2A */  OpCode ("ADD A,R2",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 2B */  OpCode ("ADD A,R3",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 2C */  OpCode ("ADD A,R4",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 2D */  OpCode ("ADD A,R5",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 2E */  OpCode ("ADD A,R6",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 2F */  OpCode ("ADD A,R7",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 30 */  OpCode ("JNB %s,%s",OpCode::bit,OpCode::reladdr,OpCode::bitDir,OpCode::dir,OpCode::noParg,3,1),
  /* 31 */  OpCode ("ACALL %s",OpCode::paddr81,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 32 */  OpCode ("RETI",OpCode::fixedarg,OpCode::noarg,OpCode::noParg,OpCode::noParg,OpCode::noParg,1,2),
  /* 33 */  OpCode ("RLC A",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::noParg,OpCode::noParg,1,1),
  /* 34 */  OpCode ("ADDC A,#%s",OpCode::imm,OpCode::noarg,OpCode::aReg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 35 */  OpCode ("ADDC A,%s",OpCode::data,OpCode::noarg,OpCode::aReg,OpCode::dir,OpCode::noParg,2,1),
  /* 36 */  OpCode ("ADDC A,@R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 37 */  OpCode ("ADDC A,@R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 38 */  OpCode ("ADDC A,R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 39 */  OpCode ("ADDC A,R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 3A */  OpCode ("ADDC A,R2",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 3B */  OpCode ("ADDC A,R3",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 3C */  OpCode ("ADDC A,R4",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 3D */  OpCode ("ADDC A,R5",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 3E */  OpCode ("ADDC A,R6",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 3F */  OpCode ("ADDC A,R7",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 40 */  OpCode ("JC %s",OpCode::reladdr,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 41 */  OpCode ("AJMP %s",OpCode::paddr82,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 42 */  OpCode ("ORL %s,A",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::aReg,OpCode::noParg,2,1),
  /* 43 */  OpCode ("ORL %s,#%s",OpCode::data,OpCode::imm,OpCode::dir,OpCode::Pimm,OpCode::noParg,3,1),
  /* 44 */  OpCode ("ORL A,#%s",OpCode::imm,OpCode::noarg,OpCode::aReg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 45 */  OpCode ("ORL A,%s",OpCode::data,OpCode::noarg,OpCode::aReg,OpCode::dir,OpCode::noParg,2,1),
  /* 46 */  OpCode ("ORL A,@R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 47 */  OpCode ("ORL A,@R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 48 */  OpCode ("ORL A,R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 49 */  OpCode ("ORL A,R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 4A */  OpCode ("ORL A,R2",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 4B */  OpCode ("ORL A,R3",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 4C */  OpCode ("ORL A,R4",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 4D */  OpCode ("ORL A,R5",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 4E */  OpCode ("ORL A,R6",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 4F */  OpCode ("ORL A,R7",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 50 */  OpCode ("JNC %s",OpCode::reladdr,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 51 */  OpCode ("ACALL %s",OpCode::paddr82,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 52 */  OpCode ("ANL %s,A",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::aReg,OpCode::noParg,2,1),
  /* 53 */  OpCode ("ANL %s,#%s",OpCode::data,OpCode::imm,OpCode::dir,OpCode::Pimm,OpCode::noParg,3,2),
  /* 54 */  OpCode ("ANL A,#%s",OpCode::imm,OpCode::noarg,OpCode::aReg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 55 */  OpCode ("ANL A,%s",OpCode::data,OpCode::noarg,OpCode::aReg,OpCode::dir,OpCode::noParg,2,1),
  /* 56 */  OpCode ("ANL A,@R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 57 */  OpCode ("ANL A,@R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 58 */  OpCode ("ANL A,R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 59 */  OpCode ("ANL A,R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 5A */  OpCode ("ANL A,R2",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 5B */  OpCode ("ANL A,R3",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 5C */  OpCode ("ANL A,R4",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 5D */  OpCode ("ANL A,R5",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 5E */  OpCode ("ANL A,R6",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 5F */  OpCode ("ANL A,R7",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 60 */  OpCode ("JZ %s",OpCode::reladdr,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 61 */  OpCode ("AJMP %s",OpCode::paddr83,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 62 */  OpCode ("XRL %s,A",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::aReg,OpCode::noParg,2,1),
  /* 63 */  OpCode ("XRL %s,#%s",OpCode::data,OpCode::imm,OpCode::dir,OpCode::Pimm,OpCode::noParg,3,2),
  /* 64 */  OpCode ("XRL A,#%s",OpCode::imm,OpCode::noarg,OpCode::aReg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 65 */  OpCode ("XRL A,%s",OpCode::data,OpCode::noarg,OpCode::aReg,OpCode::dir,OpCode::noParg,2,1),
  /* 66 */  OpCode ("XRL A,@R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 67 */  OpCode ("XRL A,@R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 68 */  OpCode ("XRL A,R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 69 */  OpCode ("XRL A,R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 6A */  OpCode ("XRL A,R2",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 6B */  OpCode ("XRL A,R3",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 6C */  OpCode ("XRL A,R4",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 6D */  OpCode ("XRL A,R5",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 6E */  OpCode ("XRL A,R6",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 6F */  OpCode ("XRL A,R7",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 70 */  OpCode ("JNZ %s",OpCode::reladdr,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 71 */  OpCode ("ACALL %s",OpCode::paddr83,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 72 */  OpCode ("ORL C,%s",OpCode::bit,OpCode::noarg,OpCode::cReg,OpCode::bitDir,OpCode::noParg,2,2),
  /* 73 */  OpCode ("JMP @A+DPTR",OpCode::fixedarg,OpCode::noarg,OpCode::idxDptr,OpCode::noParg,OpCode::noParg,1,2),
  /* 74 */  OpCode ("MOV A,#%s",OpCode::imm,OpCode::noarg,OpCode::aReg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 75 */  OpCode ("MOV %s,#%s",OpCode::data,OpCode::imm,OpCode::dir,OpCode::Pimm,OpCode::noParg,3,1),
  /* 76 */  OpCode ("MOV @R0,#%s",OpCode::imm,OpCode::noarg,OpCode::indReg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 77 */  OpCode ("MOV @R1,#%s",OpCode::imm,OpCode::noarg,OpCode::indReg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 78 */  OpCode ("MOV R0,#%s",OpCode::imm,OpCode::noarg,OpCode::reg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 79 */  OpCode ("MOV R1,#%s",OpCode::imm,OpCode::noarg,OpCode::reg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 7A */  OpCode ("MOV R2,#%s",OpCode::imm,OpCode::noarg,OpCode::reg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 7B */  OpCode ("MOV R3,#%s",OpCode::imm,OpCode::noarg,OpCode::reg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 7C */  OpCode ("MOV R4,#%s",OpCode::imm,OpCode::noarg,OpCode::reg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 7D */  OpCode ("MOV R5,#%s",OpCode::imm,OpCode::noarg,OpCode::reg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 7E */  OpCode ("MOV R6,#%s",OpCode::imm,OpCode::noarg,OpCode::reg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 7F */  OpCode ("MOV R7,#%s",OpCode::imm,OpCode::noarg,OpCode::reg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 80 */  OpCode ("SJMP %s",OpCode::reladdr,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 81 */  OpCode ("AJMP %s",OpCode::paddr84,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 82 */  OpCode ("ANL C,%s",OpCode::bit,OpCode::noarg,OpCode::cReg,OpCode::bitDir,OpCode::noParg,2,2),
  /* 83 */  OpCode ("MOVC A,@A+PC",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::idxPc,OpCode::noParg,1,2),
  /* 84 */  OpCode ("DIV AB",OpCode::noarg,OpCode::noarg,OpCode::bReg,OpCode::noParg,OpCode::noParg,1,4),
  /* 85 */  OpCode ("MOV %s,%s",OpCode::data,OpCode::data,OpCode::dir,OpCode::dir,OpCode::noParg,3,2),
  /* 86 */  OpCode ("MOV %s,@R0",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::indReg,OpCode::noParg,2,2),
  /* 87 */  OpCode ("MOV %s,@R1",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::indReg,OpCode::noParg,2,2),
  /* 88 */  OpCode ("MOV %s,R0",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::reg,OpCode::noParg,2,2),
  /* 89 */  OpCode ("MOV %s,R1",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::reg,OpCode::noParg,2,2),
  /* 8A */  OpCode ("MOV %s,R2",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::reg,OpCode::noParg,2,2),
  /* 8B */  OpCode ("MOV %s,R3",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::reg,OpCode::noParg,2,2),
  /* 8C */  OpCode ("MOV %s,R4",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::reg,OpCode::noParg,2,2),
  /* 8D */  OpCode ("MOV %s,R5",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::reg,OpCode::noParg,2,2),
  /* 8E */  OpCode ("MOV %s,R6",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::reg,OpCode::noParg,2,2),
  /* 8F */  OpCode ("MOV %s,R7",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::reg,OpCode::noParg,2,2),
  /* 90 */  OpCode ("MOV DPTR,#%s",OpCode::imm16,OpCode::noarg,OpCode::dptrReg,OpCode::Pimm,OpCode::noParg,3,2),
  /* 91 */  OpCode ("ACALL %s",OpCode::paddr84,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* 92 */  OpCode ("MOV %s,C",OpCode::bit,OpCode::noarg,OpCode::bitDir,OpCode::cReg,OpCode::noParg,2,2),
  /* 93 */  OpCode ("MOVC A,@A+DPTR",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::idxDptr,OpCode::noParg,1,2),
  /* 94 */  OpCode ("SUBB A,#%s",OpCode::imm,OpCode::noarg,OpCode::aReg,OpCode::Pimm,OpCode::noParg,2,1),
  /* 95 */  OpCode ("SUBB A,%s",OpCode::data,OpCode::noarg,OpCode::aReg,OpCode::dir,OpCode::noParg,2,1),
  /* 96 */  OpCode ("SUBB A,@R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 97 */  OpCode ("SUBB A,@R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* 98 */  OpCode ("SUBB A,R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 99 */  OpCode ("SUBB A,R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 9A */  OpCode ("SUBB A,R2",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 9B */  OpCode ("SUBB A,R3",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 9C */  OpCode ("SUBB A,R4",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 9D */  OpCode ("SUBB A,R5",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 9E */  OpCode ("SUBB A,R6",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* 9F */  OpCode ("SUBB A,R7",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* A0 */  OpCode ("ORL C,/%s",OpCode::bit,OpCode::noarg,OpCode::cReg,OpCode::notBitDir,OpCode::noParg,2,2),
  /* A1 */  OpCode ("AJMP %s",OpCode::paddr85,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* A2 */  OpCode ("MOV C,%s",OpCode::bit,OpCode::noarg,OpCode::cReg,OpCode::bitDir,OpCode::noParg,2,1),
  /* A3 */  OpCode ("INC DPTR",OpCode::noarg,OpCode::noarg,OpCode::dptrReg,OpCode::noParg,OpCode::noParg,1,2),
  /* A4 */  OpCode ("MUL AB",OpCode::noarg,OpCode::noarg,OpCode::bReg,OpCode::noParg,OpCode::noParg,1,4),
  /* A5 */  OpCode ("???",OpCode::noarg,OpCode::noarg,OpCode::noParg,OpCode::noParg,OpCode::noParg,1,1),
  /* A6 */  OpCode ("MOV @R0,%s",OpCode::data,OpCode::noarg,OpCode::indReg,OpCode::dir,OpCode::noParg,2,2),
  /* A7 */  OpCode ("MOV @R1,%s",OpCode::data,OpCode::noarg,OpCode::indReg,OpCode::dir,OpCode::noParg,2,2),
  /* A8 */  OpCode ("MOV R0,%s",OpCode::data,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* A9 */  OpCode ("MOV R1,%s",OpCode::data,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* AA */  OpCode ("MOV R2,%s",OpCode::data,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* AB */  OpCode ("MOV R3,%s",OpCode::data,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* AC */  OpCode ("MOV R4,%s",OpCode::data,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* AD */  OpCode ("MOV R5,%s",OpCode::data,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* AE */  OpCode ("MOV R6,%s",OpCode::data,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* AF */  OpCode ("MOV R7,%s",OpCode::data,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* B0 */  OpCode ("ANL C,/%s",OpCode::bit,OpCode::noarg,OpCode::cReg,OpCode::notBitDir,OpCode::noParg,2,2),
  /* B1 */  OpCode ("ACALL %s",OpCode::paddr85,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* B2 */  OpCode ("CPL %s",OpCode::bit,OpCode::noarg,OpCode::bitDir,OpCode::noParg,OpCode::noParg,2,1),
  /* B3 */  OpCode ("CPL C",OpCode::noarg,OpCode::noarg,OpCode::cReg,OpCode::noParg,OpCode::noParg,1,1),
  /* B4 */  OpCode ("CJNE A,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::aReg,OpCode::Pimm,OpCode::dir,3,2),
  /* B5 */  OpCode ("CJNE A,%s,%s",OpCode::data,OpCode::reladdr,OpCode::aReg,OpCode::dir,OpCode::dir,3,2),
  /* B6 */  OpCode ("CJNE @R0,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::indReg,OpCode::Pimm,OpCode::dir,3,2),
  /* B7 */  OpCode ("CJNE @R1,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::indReg,OpCode::Pimm,OpCode::dir,3,2),
  /* B8 */  OpCode ("CJNE R0,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::reg,OpCode::Pimm,OpCode::dir,3,2),
  /* B9 */  OpCode ("CJNE R1,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::reg,OpCode::Pimm,OpCode::dir,3,2),
  /* BA */  OpCode ("CJNE R2,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::reg,OpCode::Pimm,OpCode::dir,3,2),
  /* BB */  OpCode ("CJNE R3,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::reg,OpCode::Pimm,OpCode::dir,3,2),
  /* BC */  OpCode ("CJNE R4,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::reg,OpCode::Pimm,OpCode::dir,3,2),
  /* BD */  OpCode ("CJNE R5,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::reg,OpCode::Pimm,OpCode::dir,3,2),
  /* BE */  OpCode ("CJNE R6,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::reg,OpCode::Pimm,OpCode::dir,3,2),
  /* BF */  OpCode ("CJNE R7,#%s,%s",OpCode::imm,OpCode::reladdr,OpCode::reg,OpCode::Pimm,OpCode::dir,3,2),
  /* C0 */  OpCode ("PUSH %s",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* C1 */  OpCode ("AJMP %s",OpCode::paddr86,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* C2 */  OpCode ("CLR %s",OpCode::bit,OpCode::noarg,OpCode::bitDir,OpCode::noParg,OpCode::noParg,2,1),
  /* C3 */  OpCode ("CLR C",OpCode::noarg,OpCode::noarg,OpCode::cReg,OpCode::noParg,OpCode::noParg,1,1),
  /* C4 */  OpCode ("SWAP A",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::noParg,OpCode::noParg,1,1),
  /* C5 */  OpCode ("XCH A,%s",OpCode::data,OpCode::noarg,OpCode::aReg,OpCode::dir,OpCode::noParg,2,1),
  /* C6 */  OpCode ("XCH A,@R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* C7 */  OpCode ("XCH A,@R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* C8 */  OpCode ("XCH A,R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* C9 */  OpCode ("XCH A,R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* CA */  OpCode ("XCH A,R2",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* CB */  OpCode ("XCH A,R3",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* CC */  OpCode ("XCH A,R4",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* CD */  OpCode ("XCH A,R5",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* CE */  OpCode ("XCH A,R6",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* CF */  OpCode ("XCH A,R7",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* D0 */  OpCode ("POP %s",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* D1 */  OpCode ("ACALL %s",OpCode::paddr86,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* D2 */  OpCode ("SETB %s",OpCode::bit,OpCode::noarg,OpCode::bitDir,OpCode::noParg,OpCode::noParg,2,1),
  /* D3 */  OpCode ("SETB C",OpCode::noarg,OpCode::noarg,OpCode::cReg,OpCode::noParg,OpCode::noParg,1,1),
  /* D4 */  OpCode ("DA A",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::noParg,OpCode::noParg,1,1),
  /* D5 */  OpCode ("DJNZ %s,%s",OpCode::data,OpCode::reladdr,OpCode::dir,OpCode::dir,OpCode::noParg,3,2),
  /* D6 */  OpCode ("XCHD A,@R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* D7 */  OpCode ("XCHD A,@R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* D8 */  OpCode ("DJNZ R0,%s",OpCode::reladdr,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* D9 */  OpCode ("DJNZ R1,%s",OpCode::reladdr,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* DA */  OpCode ("DJNZ R2,%s",OpCode::reladdr,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* DB */  OpCode ("DJNZ R3,%s",OpCode::reladdr,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* DC */  OpCode ("DJNZ R4,%s",OpCode::reladdr,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* DD */  OpCode ("DJNZ R5,%s",OpCode::reladdr,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* DE */  OpCode ("DJNZ R6,%s",OpCode::reladdr,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* DF */  OpCode ("DJNZ R7,%s",OpCode::reladdr,OpCode::noarg,OpCode::reg,OpCode::dir,OpCode::noParg,2,2),
  /* E0 */  OpCode ("MOVX A,@DPTR",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indDptr,OpCode::noParg,1,2),
  /* E1 */  OpCode ("AJMP %s",OpCode::paddr87,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* E2 */  OpCode ("MOVX A,@R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,2),
  /* E3 */  OpCode ("MOVX A,@R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,3),
  /* E4 */  OpCode ("CLR A",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::noParg,OpCode::noParg,1,1),
  /* E5 */  OpCode ("MOV A,%s",OpCode::data,OpCode::noarg,OpCode::aReg,OpCode::dir,OpCode::noParg,2,1),
  /* E6 */  OpCode ("MOV A,@R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* E7 */  OpCode ("MOV A,@R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::indReg,OpCode::noParg,1,1),
  /* E8 */  OpCode ("MOV A,R0",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* E9 */  OpCode ("MOV A,R1",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* EA */  OpCode ("MOV A,R2",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* EB */  OpCode ("MOV A,R3",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* EC */  OpCode ("MOV A,R4",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* ED */  OpCode ("MOV A,R5",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* EE */  OpCode ("MOV A,R6",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* EF */  OpCode ("MOV A,R7",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::reg,OpCode::noParg,1,1),
  /* F0 */  OpCode ("MOVX @DPTR,A",OpCode::noarg,OpCode::noarg,OpCode::indDptr,OpCode::aReg,OpCode::noParg,1,2),
  /* F1 */  OpCode ("ACALL %s",OpCode::paddr87,OpCode::noarg,OpCode::dir,OpCode::noParg,OpCode::noParg,2,2),
  /* F2 */  OpCode ("MOVX @R0,A",OpCode::noarg,OpCode::noarg,OpCode::indReg,OpCode::aReg,OpCode::noParg,1,2),
  /* F3 */  OpCode ("MOVX @R1,A",OpCode::noarg,OpCode::noarg,OpCode::indReg,OpCode::aReg,OpCode::noParg,1,2),
  /* F4 */  OpCode ("CPL A",OpCode::noarg,OpCode::noarg,OpCode::aReg,OpCode::noParg,OpCode::noParg,1,1),
  /* F5 */  OpCode ("MOV %s,A",OpCode::data,OpCode::noarg,OpCode::dir,OpCode::aReg,OpCode::noParg,2,1),
  /* F6 */  OpCode ("MOV @R0,A",OpCode::noarg,OpCode::noarg,OpCode::indReg,OpCode::aReg,OpCode::noParg,1,1),
  /* F7 */  OpCode ("MOV @R1,A",OpCode::noarg,OpCode::noarg,OpCode::indReg,OpCode::aReg,OpCode::noParg,1,1),
  /* F8 */  OpCode ("MOV R0,A",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::aReg,OpCode::noParg,1,1),
  /* F9 */  OpCode ("MOV R1,A",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::aReg,OpCode::noParg,1,1),
  /* FA */  OpCode ("MOV R2,A",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::aReg,OpCode::noParg,1,1),
  /* FB */  OpCode ("MOV R3,A",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::aReg,OpCode::noParg,1,1),
  /* FC */  OpCode ("MOV R4,A",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::aReg,OpCode::noParg,1,1),
  /* FD */  OpCode ("MOV R5,A",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::aReg,OpCode::noParg,1,1),
  /* FE */  OpCode ("MOV R6,A",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::aReg,OpCode::noParg,1,1),
  /* FF */  OpCode ("MOV R7,A",OpCode::noarg,OpCode::noarg,OpCode::reg,OpCode::aReg,OpCode::noParg,1,1),
};
