/***************************************************************************
 *  editbuffer.h
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _EDITBUFFER_H
#define _EDITBUFFER_H

class EditWindow;


struct undo_t {
  enum undo_op_t { UndoDelete = 1, UndoInsert, UndoReplace };
	char *text;							/* text to be inserted or deleted */
	int start;								/* starts at this position */
	int end;									/* ends at this position */
	int changed;							/* doc changed status at this undo node */
	undo_op_t op;						/* action to execute */
};

class B_G_List {
  GList *list;
  GList *tail;
public :
  B_G_List() { list = 0; }
  void append (undo_t* mydata);
  int empty () { return list == NULL; };
  undo_t* remove_link ();
  undo_t* data() { return (undo_t*)list->data; };
  undo_t* tdata() { return (undo_t*)tail->data; };
  GList* glist() const { return list; }
};

class EditBuffer : public Gtk::TextBuffer {
public:
	enum undo_redo { c_undo, c_redo };
	enum searchFrom {fromStart, fromCursor, selectOnly}; 
protected:
	EditWindow *parent;
	char *filename;
	bool changedFlag;
	bool editing;

	// Funzioni undo/redo da GNP
	B_G_List undolist;	/* list of actions for undo */
	B_G_List redolist;	/* list of actions for redo */
	guint	numundos;	/* number of undos */

	void disable_before_redo();
	void enable_after_redo();
	void undo_replace(undo_t *undo);
	void undo_list_head_chop(B_G_List &list);
	void undo_count_decr();
	void undo_count_incr();

	void undo_list_add(char *text, int start, int end, undo_t::undo_op_t op);
	void undo_list_delete(B_G_List &list);
	// Fine Funzioni undo/redo da GNP

	int strncstr (const std::string *haystack, const std::string *needle, 
    	int caseSensitive);
	void placeCursor (int pos);

protected:
	virtual void on_changed ();
	virtual void on_insert (const Gtk::TextBuffer::iterator& pos, const Glib::ustring& text, int bytes);
	virtual void on_erase (const Gtk::TextBuffer::iterator& range_begin, const Gtk::TextBuffer::iterator& range_end);
public:

    static Glib::RefPtr<EditBuffer> create(EditWindow *p) {
        return Glib::RefPtr<EditBuffer>(new EditBuffer(p));
    }

	EditBuffer (EditWindow *p) : TextBuffer()  { parent = p; 
		changedFlag = false; editing = true;   filename = NULL;};

	bool search_execute(std::string const *find, std::string const *replace,searchFrom fromCursor, 
		bool caseSensitive, bool regExp, int &endSearch);
	int isChanged () { return changedFlag; };
	void noChanged () { setChanged (false); };
	void setChanged (bool c);
	int file_open_execute (char *file);
	int file_save_execute(const char *filename);
	const char *getFilename () { return filename; };
	void setFilename (char *f) { if (filename) { delete filename;}; filename = f;};
	char const *get_title ();
	void undo_redo_common (undo_redo which);
	void undoAll () { while (!undolist.empty ()) { undo_redo_common (c_undo); } };
	void redoAll () { while (redolist.empty ()) { undo_redo_common (c_redo); } };
	void emptyUndoRedo () {  if (!undolist.empty ()) { undo_list_delete (undolist); }
		if (!redolist.empty ()) { undo_list_delete (redolist); } };
};

#endif /* _EDITBUFFER_H */
