/***************************************************************************
 *  memorypanel.h
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _MEMORYPANEL_H
#define _MEMORYPANEL_H

#ifndef _MEMORY_H
#include "memory.h"
#endif

class Pcmon;
class Sfr;
class XDataMemory;
	
/* ******************************************************************** *
 * Header dell'oggetto GTK+ sottostante									*
 * ******************************************************************** */
#include <gtk/gtkdrawingarea.h> 
// #include <gtk/gtkmarshal.h>
// #include <gtkmarshalers.h>
G_BEGIN_DECLS
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define MEMORYPANEL(obj)          GTK_CHECK_CAST (obj, memorypanel_get_type (), Memorypanel)
#define MEMORYPANEL_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, memorypanel_get_type (), MemorypanelClass)
#define IS_MEMORYPANEL(obj)       GTK_CHECK_TYPE (obj, memorypanel_get_type ())


typedef struct _Memorypanel       Memorypanel;
typedef struct _MemorypanelClass  MemorypanelClass;

struct _Memorypanel
{
  GtkDrawingArea drawingarea;
	
  GtkAdjustment *hadjustment;
  GtkAdjustment *vadjustment;
  guint width;
  guint height;

};

struct _MemorypanelClass
{
  GtkDrawingAreaClass parent_class;

  void  (*set_scroll_adjustments)   (Memorypanel	    *memorypanel,
				     GtkAdjustment  *hadjustment,
				     GtkAdjustment  *vadjustment);


  void (* memorypanel) (Memorypanel *mp);
};

guint          memorypanel_get_type        (void);

#ifdef __cplusplus
}
#endif /* __cplusplus */

G_END_DECLS
/* ******************************************************************** *
 * Fine Header dell'oggetto GTK+ sottostante							*
 * ******************************************************************** */
/* ************************************************************************ *
 * Occorre creare il wrapper per il nuovo oggetto GTK+ "Memorypanel"		*
 * che e' una 'scrolable drawing area '										*
 * ************************************************************************ */

class MemoryPanel;
	
class MemoryPanel_Class : public Glib::Class
{
public:
#ifndef DOXYGEN_SHOULD_SKIP_THIS
  typedef MemoryPanel CppObjectType;
  typedef Memorypanel BaseObjectType;
  typedef MemorypanelClass BaseClassType;
  typedef Gtk::DrawingArea_Class CppClassParent;
  typedef GtkContainerClass BaseClassParent;

  friend class Layout;
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

  const Glib::Class& init();

  static void class_init_function(void* g_class, void* class_data);

  static Glib::ObjectBase* wrap_new(GObject*);

protected:

  //Callbacks (default signal handlers):
  //These will call the *_impl member methods, which will then call the existing default signal callbacks, if any.
  //You could prevent the original default signal handlers being called by overriding the *_impl method.
  static void set_scroll_adjustments_callback(Memorypanel* self, GtkAdjustment* p0, GtkAdjustment* p1);

  //Callbacks (virtual functions):
};
/* ************************************************************************ *
 * Fine del wrapper per il nuovo oggetto GTK+ "Memorypanel"					*
 * ************************************************************************ */

class MemoryPanel : public Gtk::DrawingArea, public Updatable {
/* Derivato da Layout - classe */
public:
#ifndef DOXYGEN_SHOULD_SKIP_THIS
  typedef MemoryPanel CppObjectType;
  typedef MemoryPanel_Class CppClassType;
  typedef Memorypanel BaseObjectType;
  typedef MemorypanelClass BaseClassType;
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

  virtual ~MemoryPanel();

private:
  friend class MemoryPanel_Class;
  static CppClassType memorypanel_class_;
  
public:
#ifndef DOXYGEN_SHOULD_SKIP_THIS
  static GType get_type()      G_GNUC_CONST;
  static GType get_base_type() G_GNUC_CONST;
#endif

  ///Provides access to the underlying C GtkObject.
  MemoryPanel*       gobj()       { return reinterpret_cast<MemoryPanel*>(gobject_); }

  ///Provides access to the underlying C GtkObject.
  const MemoryPanel* gobj() const { return reinterpret_cast<MemoryPanel*>(gobject_); }
protected:
  explicit MemoryPanel(Memorypanel* castitem);

/* Fine Derivato da Layout - classe */
private:
  SigC::Connection cursorTimer;

	virtual void showLine (Glib::RefPtr<Gdk::Window> win, unsigned line, int y);
	virtual void key (int k);
	virtual void setScrollSizes (int w, int h);
	virtual void setPosition (int x);

	virtual bool on_button_press_event (GdkEventButton *);

	void xChanged (void);
	void yChanged (void);

	virtual unsigned curPos ();
protected:
	Gtk::Adjustment *HAdj;
	Gtk::Adjustment *VAdj;
	Pcmon *parent;
	unsigned curX, curY, Zone;
	bool low;

	Pango::FontDescription *font;
	int text_w;
	int text_h;
	bool curOn;

	Memory *data;
	unsigned xOffs, yOffs;
	static int const border = 5;
	unsigned curW, curH;
	
	virtual bool tick();
	/* Redraw the screen */
	virtual bool on_expose_event (GdkEventExpose *event);
	virtual bool on_configure_event (GdkEventConfigure *event);
	virtual bool on_focus_in_event (GdkEventFocus *e);
    virtual bool on_focus_out_event (GdkEventFocus *e);
	virtual void on_set_scroll_adjustments(Gtk::Adjustment* hadj, Gtk::Adjustment* vadj);

public:
	void reload () { data -> reload (); };
	virtual void redraw () {  queue_draw (); };
	virtual bool on_key_press_event (GdkEventKey* p1);
	virtual void goTo (unsigned where);
	
	// Gestione Segnale
	Glib::SignalProxy2< void,Gtk::Adjustment*,Gtk::Adjustment* > signal_set_scroll_adjustments();
	// Fine Gestione Segnale

	MemoryPanel (Memory *d, Pcmon *p);
};

MemoryPanel* wrap(Memorypanel* object, bool take_copy = false);

#endif /* _MEMORYPANEL_H */
