/***************************************************************************
 *  ppi.h
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#ifndef PPI_H
#define PPI_H

#ifndef _PPIINTERFACE_H
#include "ppiinterface.h"
#endif

enum image { redled = 0, greenled = 1, yellowled = 2, redbutton = 3,  greenbutton = 4, 
  yellowbutton = 5, dipswitch = 6 };
enum behave { led, button, dip };
enum port { portA = 0, portB = 1, portC = 2, P1 = 4, inportA = 5, inportC = 7 };

struct bobject {
	int y,x;
	int mask;
	port command;
	image what;
	behave how;
};

class ppi;

class binary {
protected:
	Glib::RefPtr<Gdk::Pixmap> off;
	Glib::RefPtr<Gdk::Pixmap> on;
	int x,y;
	int width, height; 
	int *port;
	int mask;
public:
	binary (int const X, int const Y, Glib::RefPtr<Gdk::Pixmap> Off, Glib::RefPtr<Gdk::Pixmap> On, int *Port, int const Mask);
	void show (Glib::RefPtr<Gdk::Pixmap> win, Glib::RefPtr<Gdk::GC> gc);
	virtual void click (int press, int x, int y, ppi *parent) {};
	void redraw (ppi *parent);
	virtual ~binary () {};
};

class Button : public binary {
public:
	Button (const int X, const int Y, Glib::RefPtr<Gdk::Pixmap> Off, Glib::RefPtr<Gdk::Pixmap> On, int *Port, const int Mask) :
		binary (X, Y, Off, On, Port, Mask) {};
	virtual void click (int press, int x, int y, ppi *parent);
};

class Dipsw : public binary {
public:
	Dipsw (const int X, const int Y, Glib::RefPtr<Gdk::Pixmap> Off, Glib::RefPtr<Gdk::Pixmap> On, int *Port, const int Mask) :
		binary (X, Y, Off, On, Port, Mask) {};
	virtual void click (int press, int x, int y, ppi *parent);
};

class ppi  : public Gtk::DrawingArea, public ppiinterface
{
	int text_h;
	int text_a;
	int text_w;

	volatile bool toRedraw;

	int ports [9];
	int outports [4];

	/* Backing pixmap for drawing area */
	Glib::RefPtr<Gdk::Pixmap> pixmap;

	Glib::RefPtr<Gdk::Pixmap> images [7][2];
	static const int Width = 275;
	static const int Height = 130;
	static const int nobjects = 42;
	static const bobject inits [nobjects];
  
	binary *disps [nobjects];

	bool on_configure_event (GdkEventConfigure *event);

	/* Redraw the screen from the backing pixmap */
	bool on_expose_event (GdkEventExpose *event);

	Glib::RefPtr<Gdk::Pixmap> makeIcon (char **icon);
	void redrawPorts (int emulating);

public:
	ppi ();
	void Out (int address, int datum, int emulating);
	unsigned In (int address);
	virtual void P1Out (int datum, int emulating);
	virtual void redraw (int x = 0, int y = 0, int w = 0, int h = 0);
    virtual void redrawIfChanged();
	bool on_button_press_event (GdkEventButton *);
	bool on_button_release_event (GdkEventButton *);
};

#endif
