/***************************************************************************
 *  sfrmemory.h
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _SFRMEMORY_H
#define _SFRMEMORY_H

#ifndef MEMORY_H
#include "memory.h"
#endif

class HRegister {
protected:
	char set [2];	// Caratteri del comando per impostare il registro
	Updatable *r;
	int is8bit;
	int value;

	enum stat_val status;
	bool toRedraw;
public:
	HRegister (int is8Bit, char *c);
	void store ();
	void setRefresh (Updatable *newR) { r = newR; };
	virtual int getValue () { return value; };
	virtual void setValue (int v) { value = v; store (); };	// Modifica il valore e lo scrive sulla scheda
	virtual HRegister &operator = (int i);						// Modifica il valore ed aggiorna campo associato
	void redraw () { if (status == emulating) { toRedraw = true; } else { r -> redraw (); } };
	void redrawIfChanged () { if (toRedraw) { toRedraw = false; r -> redraw ();} };
	void setStatus (stat_val s) { status = s;};
};

class BpHRegister;

#define NUMBP	4
class BpPool {
	BpHRegister *PC;
	BpHRegister *BPs [NUMBP];
public:
	void setRegisters ( BpHRegister *pc, BpHRegister *b0, BpHRegister *b1, BpHRegister *b2, BpHRegister *b3)
		{ PC = pc; BPs[0] = b0; BPs[1] = b1; BPs[2] = b2; BPs[3] = b3;};
	bool checkBps (unsigned v, int bpindex);
	bool isBp (unsigned v);
	bool hasBp ();
	void addBkp (unsigned PC);
	void delBkp (unsigned PC);
};

class BpHRegister : public HRegister {
	BpPool *pool;
	int bpindex;
public:
	virtual HRegister &operator = (int i);						// Modifica il valore ed aggiorna campo associato
	BpHRegister (char *c, int index) : HRegister (false, c) { pool = NULL; bpindex = index;};
	void setPool (BpPool *p) { pool = p;};
	virtual void setValue (int v) { if (pool->checkBps(v, bpindex)) { value = v; store (); } };	// Modifica il valore e lo scrive sulla scheda
};

class Serial_Container;
class ppiinterface;

class SFRMemory : public Memory {
	int SendSerial;
	
	Serial_Container *s;
	ppiinterface *PPI;

	int prgChanging;

	HRegister sp;
	HRegister a;
	HRegister b;
	HRegister psw;
	HRegister dptr;

	BpPool pool;
	BpHRegister pc;
	BpHRegister b0;
	BpHRegister b1;
	BpHRegister b2;
	BpHRegister b3;

public:
	SFRMemory (Serial_Container *c_s);
	virtual void reload ();					// Ricarica l'intero contenuto dello spazio
	virtual void saveValue (int pos);	// Registra il dato sulla scheda
	virtual void redraw ();
	virtual void redrawIfChanged ();
	virtual void setStatus (stat_val s);

	bool checkBps (unsigned v) { return pool.checkBps (v, -1); };
	bool isBp (unsigned v) { return pool.isBp (v);};
	bool hasBp () { return pool.hasBp (); };
	bool isBrk () { return pool.isBp (pc.getValue ()); };
	void addBkp (unsigned PC) { pool.addBkp (PC); };
	void delBkp (unsigned PC) { pool.delBkp (PC); };

	void setPPI (ppiinterface *c_PPI) { PPI = c_PPI; };

	// Funzioni per gestione display
	HRegister* GetAccReg (void) { return &a; };
	HRegister* GetBReg (void) { return &b; };
	HRegister* GetPSWReg (void) { return &psw; };
	HRegister* GetSPReg (void) { return &sp; };
	HRegister* GetDPTRReg (void) { return &dptr; };
	
	HRegister* GetPcReg (void) { return &pc; };
	HRegister* GetB0Reg (void) { return &b0; };
	HRegister* GetB1Reg (void) { return &b1; };
	HRegister* GetB2Reg (void) { return &b2; };
	HRegister* GetB3Reg (void) { return &b3; };
	// Funzioni per emulazione
	void receiver (char c);
	unsigned char GetAcc (void) { return a.getValue (); };
	unsigned char GetPSW (void) { return psw.getValue (); };
	unsigned char GetSP (void) { return sp.getValue (); };
	unsigned GetDPTR (void) { return dptr.getValue (); };
	unsigned GetPc (void) { return pc.getValue (); };
	unsigned GetB0 (void) { return b0.getValue (); };
	unsigned GetB1 (void) { return b1.getValue (); };
	unsigned GetB2 (void) { return b2.getValue (); };
	unsigned GetB3 (void) { return b3.getValue (); };

	void SetAcc (unsigned value) { a = value;};
	void SetBreg (unsigned value) { b = value;};
	void SetPSW (unsigned value) { psw = value;};
	void SetSP (unsigned value) { sp = value;};
	void SetDPTR (unsigned value) { dptr = value;};
	void SetPc (unsigned value) { pc = value;};
	void SetB0 (unsigned value) { b0 = value;};
	void SetB1 (unsigned value) { b1 = value;};
	void SetB2 (unsigned value) { b2 = value;};
	void SetB3 (unsigned value) { b3 = value;};

	int testSerial () { if (SendSerial) { if (!--SendSerial) return -1; } return 0;};

	unsigned char GetSFR (int reg);
	void SetSFR (int reg,unsigned v);
	void store (int where);
};
#endif /* _SFRMEMORY_H */
