/***************************************************************************
 *  data.cc
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include <glib/gmacros.h>
#include <glib.h>
#include <gtkmm.h>
#include <libgnomemm.h>

#include <iostream>	// Per la stampa dell'eventuale errore

#ifndef MEMORYPANEL_H
#include "memorypanel.h"
#endif

#include "data.h"
#include "pcmon.h"

void PcData::toolbarBuild (Pcmon *parent)
{
  //Create actions for menus and toolbars:
  m_refActionGroup = Gtk::ActionGroup::create();

  m_refActionGroup->add( Gtk::Action::create("Esegui", Gtk::Stock::EXECUTE),
  	sigc::mem_fun(*parent,&Pcmon::menuRun) );
  m_refActionGroup->add( Gtk::Action::create("EseguiInizio", Gtk::StockID("pcmon_btn_runstart")),
  	sigc::mem_fun(*parent,&Pcmon::menuRunStart) );
  m_refActionGroup->add( Gtk::Action::create("PassoSingolo", Gtk::StockID("pcmon_btn_singlestep")),
  	sigc::mem_fun(*parent,&Pcmon::menuStep) );

  m_refActionGroup->add( Gtk::Action::create("Ricarica", Gtk::Stock::REFRESH),
  	sigc::mem_fun(*this,&PcData::menuRefresh) );

  m_refUIManager = Gtk::UIManager::create();
  m_refUIManager->insert_action_group(m_refActionGroup);
  add_accel_group(m_refUIManager->get_accel_group());

  try
  {
    /* Crea l'immagine per tutti i menu (principale e popup) e le toolbar */
    Glib::ustring ui_info =
        "<ui>"
        "  <toolbar  name='ToolBar'>"
        "    <toolitem action='Esegui'/>"
        "    <toolitem action='EseguiInizio'/>"
        "    <toolitem action='PassoSingolo'/>"
        "    <separator/>"
		"    <toolitem action='Ricarica'/>"
        "  </toolbar>"
        "</ui>";

    m_refUIManager->add_ui_from_string(ui_info);
  }
  catch(const Glib::Error& ex)
  {
    std::cerr << "Preparazione dei menu fallita: " <<  ex.what();
  }

  //Get the toolbar widgets, and add them to a container widget:
  Gtk::Toolbar* toolbar = (Gtk::Toolbar *) m_refUIManager->get_widget("/ToolBar") ;
  if(toolbar)
    vbox.pack_start(*toolbar, Gtk::PACK_SHRINK);
  toolbar->set_toolbar_style(Gtk::TOOLBAR_ICONS);
}

PcData::PcData (Pcmon *p, Memory *data)
{
	set_title ("Spazio Data");
	display = new MemoryPanel (data,p);
	toolbarBuild (p);
	add (vbox);

	//  dataMem.getWidget ().set_default_size (450,150);
	set_default_size (450,150);
	vbox.pack_start(base, true, true,0);
	base.add (*display);
	set_sensitive (true);
	display->grab_focus ();
	signal_key_press_event().connect(sigc::mem_fun(*display,&MemoryPanel::on_key_press_event));

	set_events (Gdk::KEY_PRESS_MASK);

	display->reload ();
	show_all ();
}
