/***************************************************************************
 *  opcode.cc
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include <glib/gmacros.h>
#include <glib.h>
#include <gtkmm.h>
#include <libgnomemm.h>

#include <stdio.h>
#include <ctype.h>

#ifndef DMEMORY_H
#include "dmemory.h"
#endif
#ifndef XMEMORY_H
#include "xmemory.h"
#endif
#ifndef MEMORY_H
#include "memory.h"
#endif
#include "opcode.h"
#include "symtabs.h"
#include "sfrnames.h"
#ifndef _I8052_H
#include "i8052.h"
#endif
 

DataMemory *OpCode::Data;
XDataMemory *OpCode::Xdata;
Memory *OpCode::Code;

OpCode::ErrVals OpCode::ErrCode;

//	enum ErrVals { noerr, toobig, toofar, isbitaddr, noopcode, noop, numerr, charconst, numerr2, nosfr, parserr, dupsym, nosym, 
//		strerr, idxerr, indreg, inderr, bitnumerr, notbitadressable, parnotbal, illbitaddrop, divbyzero, illoper };
char *OpCode::ErrStrs [] = {
	"Nessun Errore",											/* noerr */
	"Numero troppo grosso",								/* toobig */
	"Destinazione troppo lontana",					/* toofar */
	"Indirizzo di Bit non appropriato",			/* isbitaddr */
	"Istruzione Sconosciuta",							/* noopcode */
	"Manca un Operando",									/* noop */
	"Numero Scorretto",										/* numerr F */
	"Costante Carattere non Terminata",			/* charconst F */
	"Numero scorretto",										/* numerr2 F */
	"Nome di SFR/simbolo ignoto",					/* nosfr */
	"Parametri sbagliati per l'istruzione",	/* parserr */
	"Simbolo duplicato",									/* dupsym */
	"Simblo non definito",								/* nosym F*/
	"stringa non terminata",								/* strerr */
	"indirizzamento indicizzato scorretto",	/* idxerr */
	"Registro scorretto per indirizzamneto indiretto", /* indreg */
	"Gli indirizzamenti indiretti possono essere solo con R0,R1 o DTR", /* inderr */
	"Numero di bit non valido (validi da .0 a .7)",	/* bitnumerr */
	"La cella di memoria DATA non e' bitadressable",	/* notbitadressable */
	"Parentesi aperta non bilanciata",			/* parnotbal */
	"Operazione non valida con indirizzo BIT",	/* illbitaddrop */
	"Divisione per zero",									/* divbyzero */
	"Operatore o formato numerico non valido",	/* illoper */
};

char * const OpCode::ErrStr (ErrVals code)
{
	if (code >= (int) (sizeof (ErrStrs) / sizeof (char *))) {
		return "Errore Sconosciuto";
	}
	return ErrStrs [code];
}


void OpCode::MkArg (char *a,int offs, unsigned At,argType Arg)
{
	a [0] = 0;
	switch (Arg)
	{
		case imm : //     Immediato
			sprintf (a,"0%02xH",GetArgVal (offs,At,Arg));
			break;
		case data : //   Data
			Syms [0].GetName (a,GetArgVal (offs,At,Arg));
			break;
		case bit : //   Bit
			Syms [1].GetName (a,GetArgVal (offs,At,Arg));
			break;
		default :
			sprintf (a,"0%04xH",GetArgVal (offs,At,Arg));
			break;
	}
}

unsigned OpCode::GetArgVal (int offs,unsigned Ad,argType Arg)
{
	switch (Arg)
	{
		case imm : //   Immediato
		case data : //   Data
		case bit : //   Bit
			return Code -> GetDato (Ad+offs) & 0xff;
		case reladdr : //    <jmp> <rel>
			{
			int off = Code -> GetDato (Ad+offs);

				if (off & 0x80) {	// Gestisce il segno
					off = (-1<<8) | off;

				}
				return Ad + off + Len;
			}
		case imm16 : //    MOV DPTR,#num
		case absaddr : //    LCALL/LJMP
			return (((unsigned) Code -> GetDato (Ad+offs)) << 8) | ((unsigned) Code -> GetDato (Ad+offs + 1));
		case paddr80 : //..0x87 (AJMP/ACALL) 0..7 -> bit 10..8, byte seguente -> byte 7..0
		case paddr81 :
		case paddr82 :
		case paddr83 :
		case paddr84 :
		case paddr85 :
		case paddr86 :
		case paddr87 :
			return (Ad & 0xf800) | ((Arg << 8) & 0x700) | Code -> GetDato (Ad+offs);
		default:
			break;
	}
	return 0;
}

unsigned char OpCode::Oper (unsigned char Opcode,unsigned char Op1,
  unsigned char Op2)
{
unsigned res,carry;
  switch (Opcode & 0xf0)
  {
    case 0x00 :  /* 0000 XXXX       INC */
      return Op2 + 1;
    case 0x10 :      /* 0001 XXXX   DEC */
      return Op2 - 1;
    case 0x20 :      /* 0010 XXXX   ADD CY-AC-OV */
      res = Op1 + Op2;
      Data -> SetBit (bCY,res & 0x100);
      Data -> SetBit (bOV,((res & 0x100) >> 1) ^ (
        ((Op1 & 0x7f) + (Op2 & 0x7f)) & 0x80));
      Data -> SetBit (bAC,((Op1 & 0xf) + (Op2 & 0xf)) & 0x10);
      return res;
    case 0x30 :      /* 0011 XXXX   ADDC CY-AC-OV */
      carry = (Data -> GetBit (bCY)) ? 1 : 0;
      res = Op1 + Op2 + carry;
      Data -> SetBit (bCY,res & 0x100);
      Data -> SetBit (bAC,((Op1 & 0xf) + (Op2 & 0xf) + carry)
        & 0x10);
      Data -> SetBit (bOV,((res & 0x100) >> 1) ^ (
        ((Op1 & 0x7f) + (Op2 & 0x7f) + carry) & 0x80));
      return res;
    case 0x40 :      /* 0100 XXXX   ORL */
      return Op1 | Op2;
    case 0x50 :      /* 0101 XXXX   ANL */
      return Op1 & Op2;
    case 0x60 :      /* 0110 XXXX   XRL */
      return Op1 ^ Op2;
    case 0x70 :      /* 0111 XXXX   MOV A, */
      return Op2;
    case 0x90 :      /* 1001 XXXX   SUBB CY-AC-OV */
      carry = (Data -> GetBit (bCY)) ? 1 : 0;
      res = Op1 - Op2 - carry;
      Data -> SetBit (bCY,res & 0x100);
      Data -> SetBit (bAC,((Op1 & 0xf) - (Op2 & 0xf) - carry)
        & 0x10);
      Data -> SetBit (bOV,((res & 0x100) >> 1) ^ (
        ((Op1 & 0x7f) - (Op2 & 0x7f) - carry) & 0x80));
      return res;
  }
  return 0;
}

unsigned OpCode::Exec (unsigned Pc)
{
unsigned DoJmp;
unsigned char Opcode = Code -> GetDato (Pc);
unsigned char Opr,Opr2;

  switch (Args[0])
  {
    case 0: // istruzioni da 1 byte
      switch (Opcode)
      {
        case 0x00 :     /*      { "NOP",0,0,1}, */
          break;
        case 0x03 :     /*      { "RR A",0,0,1}, */
          DoJmp = Data -> GetDDato (aACC) & 0xff;
          DoJmp |= (DoJmp & 0x1) ? 0x100 : 0;
          DoJmp >>= 1;
          Data -> SetDDato (aACC,DoJmp);
          break;
        case 0x04 : /*  { "INC A",0,0,1}, */
          Data -> SetDDato (aACC,Data -> GetDDato (aACC) + 1);
          break;
        case 0x13 :     /*      { "RRC A",0,0,1}, */
          DoJmp = Data -> GetDDato (aACC) & 0xff;
          DoJmp |= (Data -> GetBit (bCY)) ? 0x100 : 0;
          Data -> SetBit (bCY,DoJmp & 1);
          DoJmp >>= 1;
          Data -> SetDDato (aACC,DoJmp);
          break;
        case 0x14 :     /*      { "DEC A",0,0,1}, */
          Data -> SetDDato (aACC,Data -> GetDDato (aACC) - 1);
          break;
        case 0x23 :     /*      { "RL A",0,0,1}, */
          DoJmp = Data -> GetDDato (aACC) << 1;
          DoJmp |= (DoJmp & 0x100) ? 1 : 0;;
          Data -> SetDDato (aACC,DoJmp);
          break;
        case 0x33 : /*  { "RLC A",0,0,1}, */
          DoJmp = Data -> GetDDato (aACC) << 1;
          DoJmp |= (Data -> GetBit (bCY)) ? 1 : 0;
          Data -> SetBit (bCY,DoJmp & 0x100);
          Data -> SetDDato (aACC,DoJmp);
          break;
        case 0x83 : /*  { "MOVC A,@A+PC",0,0,1}, */
          DoJmp = (Pc + Len) & 0xffff;
          DoJmp += Data -> GetDDato (aACC) & 0xff;
          DoJmp &= 0xffff;
          Data -> SetDDato (aACC,(DoJmp >= Code -> getStart ()  && DoJmp < Code -> getEnd()) ?
            Code -> GetDato (DoJmp) : 0);
          break;
        case 0x84 :     /*      { "DIV AB",0,0,1}, */
          DoJmp = Data -> GetDDato (aACC) & 0xff;
          Opr = Data -> GetDDato (aB) & 0xff;
          Data -> SetDDato (aACC,DoJmp / Opr);
          Data -> SetDDato (aB,DoJmp % Opr);
          Data -> SetBit (bCY,0);
          Data -> SetBit (bOV,0);
          break;
        case 0x93 : /*  { "MOVC A,@A+DPTR",0,0,1}, */
          DoJmp = Data -> GetDPTR ();
          DoJmp += Data -> GetDDato (aACC) & 0xff;
          DoJmp &= 0xffff;
          Data -> SetDDato (aACC,(DoJmp >= Code -> getStart () && DoJmp < Code -> getEnd ()) ?
            Code -> GetDato (DoJmp) : 0);
          break;
        case 0xA3 : /*  { "INC DPTR",0,0,1}, */
          DoJmp = Data -> GetDDato (aDPL);
          DoJmp += 1;
          Data -> SetDDato (aDPL,DoJmp);
          if (!(DoJmp & 0xff))
          {
            Data -> SetDDato (aDPH,Data -> GetDDato (aDPH) + 1);
          }
          break;
        case 0xA4 : /*  { "MUL AB",0,0,1}, */
          DoJmp = (Data -> GetDDato (aACC) & 0xff) *
            (Data -> GetDDato (aB) & 0xff);
          Data -> SetDDato (aACC,DoJmp);
          Data -> SetDDato (aB,DoJmp >> 8);
          Data -> SetBit (bCY,0);
          Data -> SetBit (bOV,DoJmp & 0xff00);
          break;
        case 0xB3 : /*  { "CPL C",0,0,1}, */
          Data -> SetBit (bCY,!Data -> GetBit (bCY));
          break;
        case 0xC3 : /*  { "CLR C",0,0,1}, */
          Data -> SetBit (bCY,1);
          break;
        case 0xC4 : /*  { "SWAP A",0,0,1}, */
          DoJmp = Data -> GetDDato (aACC) << 4;
          Data -> SetDDato (aACC,(DoJmp & 0xf0) ||
            ((DoJmp >> 8) & 0x0f));
          break;
        case 0xD3 : /*  { "SETB C",0,0,1}, */
          Data -> SetBit (bCY,1);
          break;
        case 0xD4 : /*  { "DA A",0,0,1}, */
          DoJmp = Data -> GetDDato (aACC);
          if ((DoJmp & 0xf) > 9 ||
            Data -> GetBit (bAC))
              DoJmp += 6;
          Opr = Data -> GetBit (bCY);
          if ((DoJmp & 0xf0) > 0x90 || Opr)
            DoJmp += 0x60;
          Data -> SetDDato (aACC,DoJmp);
          Data -> SetBit (bCY,Opr || (DoJmp & 0x100));
          break;
        case 0xD6 :
        case 0xD7 : /*  { "XCHD A,@RX",0,0,1}, */
          Opr = Data -> GetRegister (Opcode & 1);
          Opr2 = Data -> GetDDato (aACC);
          DoJmp = Data -> GetDato (Opr);
          Data -> SetDDato (aACC,
            (Opr2 & 0xf0) | (DoJmp & 0xf));
          Data -> SetDato (Opr,(DoJmp & 0xf0) | (Opr2 & 0xf));
          break;
        case 0xE0 : /*  { "MOVX A,@DPTR",0,0,1}, */
          DoJmp = Data -> GetDPTR ();
          Data -> SetDDato (aACC,Xdata -> GetDato (DoJmp));
          break;
        case 0xE2 :
        case 0xE3 : /*  { "MOVX A,@RX",0,0,1}, */
          DoJmp = Data -> GetRegister (Opcode & 1) & 0xff; 
          Data -> SetDDato (aACC,Xdata -> GetDato (DoJmp));
          break;
        case 0xE4 : /*  { "CLR A",0,0,1}, */
          Data -> SetDDato (aACC,0);
          break;
        case 0xF0 : /*  { "MOVX @DPTR,A",0,0,1}, */
          DoJmp = Data -> GetDPTR ();
          Xdata -> SetDato (DoJmp,Data -> GetDDato (aACC));
          break;
        case 0xF2 : /*  { "MOVX @R0,A",0,0,1}, */
        case 0xF3 : /*  { "MOVX @R1,A",0,0,1}, */
          DoJmp = Data -> GetRegister (Opcode & 1) & 0xff;
          Xdata -> SetDato (DoJmp, Data -> GetDDato (aACC));
          break;
        case 0xF4 : /*  { "CPL A",0,0,1}, */
          Data -> SetDDato (aACC,~Data -> GetDDato (aACC));
          break;
        default :
          switch (Opcode & 0xf8)
          {
            case 0x0 : /* 06.07{ "INC @RX",0,0,1}, */
              Opr = Data -> GetRegister (Opcode & 1);
              Data -> SetDato (Opr,Data -> GetDato (Opr) + 1);
              break;
            case 0x8 : /* 08.0F{ "INC RX",0,0,1}, */
              Data -> SetRegister (Opcode,Data -> GetRegister (Opcode) + 1);
              break;
            case 0x10 : /* 16.17{ "DEC @RX",0,0,1}, */
              Opr = Data -> GetRegister (Opcode & 1);
              Data -> SetDato (Opr,Data -> GetDato (Opr) - 1);
              break;
            case 0x18 :     /* 18.1F{ "DEC RX",0,0,1}, */
              Data -> SetRegister (Opcode,Data -> GetRegister (Opcode) - 1);
              break;
            case 0x20 : /* 26.27{ "ADD A,@RX",0,0,1}, */
            case 0x30 : /* 36.37{ "ADDC A,@RX",0,0,1}, */
            case 0x40 : /* 46.47{ "ORL A,@RX",0,0,1}, */
            case 0x50 : /* 56.57{ "ANL A,@RX",0,0,1}, */
            case 0x60 : /* 66.67{ "XRL A,@RX",0,0,1}, */
            case 0x90 : /* 96.97{ "SUBB A,@RX",0,0,1}, */
              Data -> SetDDato (aACC,Oper (Opcode,Data -> GetDDato (aACC),
                Data -> GetDato (Data -> GetRegister (Opcode & 1))));
              break;
            case 0x28 : /* 28.2F{ "ADD A,RX",0,0,1}, */
            case 0x38 : /* 38.3F{ "ADDC A,RX",0,0,1}, */
            case 0x48 : /* 48.4F{ "ORL A,RX",0,0,1}, */
            case 0x58 : /* 58.5F{ "ANL A,RX",0,0,1}, */
            case 0x68 : /* 68.6F{ "XRL A,RX",0,0,1}, */
            case 0x98 : /* 98.9F{ "SUBB A,RX",0,0,1}, */
              Data -> SetDDato (aACC,Oper (Opcode,Data -> GetDDato (aACC),
                Data -> GetRegister (Opcode)));
              break;
            case 0xc0 : /* C6.C7{ "XCH A,@RX",0,0,1}, */
              Opr = Data -> GetDDato (Opcode & 1);
              DoJmp = Data -> GetDato (Opr);
              Data -> SetDato (Opr,Data -> GetDDato (aACC));
              Data -> SetDDato (aACC,DoJmp);
              break;
            case 0xE0 : /* E6.E7{ "MOV A,@RX",0,0,1}, */
              Data -> SetDDato (aACC,Data -> GetDato (
                Data -> GetRegister (Opcode & 1)));
              break;
            case 0xf0 : /* F6.F7{ "MOV @RX,A",0,0,1}, */
              Data -> SetDato (Data -> GetRegister (Opcode & 1),
                Data -> GetDDato (aACC));
              break;
            case 0xC8 : /* C8.CF{ "XCH A,RX",0,0,1}, */
              DoJmp = Data -> GetRegister (Opcode);
              Data -> SetRegister (Opcode,Data -> GetDDato (aACC));
              Data -> SetDDato (aACC,DoJmp);
              break;
            case 0xE8 : /* E8.EF{ "MOV A,RX",0,0,1}, */
              Data -> SetDDato (aACC,Data -> GetRegister (Opcode));
              break;
            case 0xf8 : /* F8.FF{ "MOV RX,A",0,0,1}, */
              Data -> SetRegister (Opcode,Data -> GetDDato (aACC));
              break;
          }
        break;
      }
      break;
    case 1 : //   Immediato
      Opr = Code -> GetDato (Pc + 1);
      if (Opcode == 0xB4)     /* { "CJNE A,#%s,%s",1,5,3}, */
      {
        if (Data -> GetDDato (aACC) != Opr)
        return GetArgVal (2,Pc,Args [1]);
      }
      else if ((Opcode & 0xfe) == 0x76)
      { /* 76.77{ "MOV @R0,#%s",1,0,2}, */
        Data -> SetDato (Data -> GetRegister (Opcode & 1),Opr);
      }
      else if ((Opcode & 0xf8) == 0x78)
      { /* 78.7F{ "MOV RX,#%s",1,0,2}, */
        Data -> SetRegister (Opcode,Opr);
      }
      else if ((Opcode & 0xfe) == 0xb6)
      { /* B6.B7{ "CJNE @R0,#%s,%s",1,5,3}, */
        if (Data -> GetDato (Data -> GetRegister (Opcode & 1)) != Opr)
          return GetArgVal (2,Pc,Args [1]);
      }
      else if ((Opcode & 0xf8) == 0xB8)
      { /* B8.BF{ "CJNE RX,#%s,%s",1,5,3}, */
        if (Data -> GetRegister (Opcode) != Opr)
          return GetArgVal (2,Pc,Args[1]);
      }
      else if ((Opcode & 0x0f) == 4)
      { /* 24   { "ADD A,#%s",1,0,2},
         * 34   { "ADDC A,#%s",1,0,2},
         * 44   { "ORL A,#%s",1,0,2},
         * 54   { "ANL A,#%s",1,0,2},
         * 64   { "XRL A,#%s",1,0,2},
         * 74   { "MOV A,#%s",1,0,2},
         * 94   { "SUBB A,#%s",1,0,2}, */
        Data -> SetDDato (aACC,Oper (Opcode,Data -> GetDDato (aACC),Opr));
      }
      break;
    case 2 : //   Data
      Opr = Code -> GetDato (Pc + 1);
      switch (Opcode)
      {
        case 0x05 : /*  { "INC %s",2,0,2}, */
          Data -> SetDDato (Opr,Data -> GetDDato (Opr) + 1);
          break;
        case 0x15 : /*  { "DEC %s",2,0,2}, */
          Data -> SetDDato (Opr,Data -> GetDDato (Opr) - 1);
          break;
        case 0x75 : /*  { "MOV %s,#%s",2,1,3}, */
          Data -> SetDDato (Opr,Code -> GetDato (Pc + 2));
          break;
        case 0x85 : /*  { "MOV %s,%s",2,2,3}, */
          Data -> SetDDato (Opr, Data -> GetDDato (Code -> GetDato (Pc + 2)));
          break;
        case 0xB5 : /*  { "CJNE A,%s,%s",2,5,3}, */
          if (Data -> GetDDato (Opr) != Data -> GetDDato (aACC))
            return GetArgVal (2,Pc,Args[1]);
          break;
        case 0xC0 : /*  { "PUSH %s",2,0,2}, */
          Data -> Push (Data -> GetDDato (Opr));
          break;
        case 0xC5 : /*  { "XCH A,%s",2,0,2}, */
          DoJmp = Data -> GetDDato (Opr);
          Data -> SetDDato (Opr,Data -> GetDDato (aACC));
          Data -> SetDDato (aACC,DoJmp);
          break;
        case 0xD0 : /*  { "POP %s",2,0,2}, */
          Data -> SetDDato (Opr,Data -> Pop ());
          break;
        case 0xD5 : /*  { "DJNZ %s,%s",2,5,3}, */
          DoJmp = Data -> GetDDato (Opr) - 1;
          Data -> SetDDato (Opr,DoJmp);
          if (DoJmp)
            return GetArgVal (2,Pc,Args[1]);
          break;
        case 0xE5 : /*  { "MOV A,%s",2,0,2}, */
          Data -> SetDDato (aACC ,Data -> GetDDato (Opr));
          break;
        case 0xF5 : /*  { "MOV %s,A",2,0,2}, */
          Data -> SetDDato (Opr,Data -> GetDDato (aACC));
          break;
        default :
          switch (Opcode & 0xF8)
          {
            case 0x80 : /* 86.87{ "MOV %s,@RX",2,0,2}, */
              Data -> SetDDato (Opr,
              Data -> GetDato (Data -> GetRegister (Opcode & 1)));
              break;
            case 0x88 : /* 88.8F{ "MOV %s,RX",2,0,2}, */
              Data -> SetDDato (Opr,Data -> GetRegister (Opcode));
              break;
            case 0xa0 : /* A6.A7{ "MOV @RX,%s",2,0,2}, */
              Data -> SetDato (Data -> GetRegister (Opcode & 1),
                Data -> GetDDato (Opr));
              break;
            case 0xa8 : /* A8.AF{ "MOV RX,%s",2,0,2}, */
              Data -> SetRegister (Opcode,Data -> GetDDato (Opr));
              break;
            default :
              switch (Opcode & 0xf)
              {
                case 0x02 : /* 42   { "ORL %s,A",2,0,2}, *
                             * 52   { "ANL %s,A",2,0,2}, *
                             * 62   { "XRL %s,A",2,0,2}, */
                  Data -> SetDDato (Opr,Oper (Opcode,Data -> GetDDato (Opr),
                    Data -> GetDDato (aACC)));
                  break;
                case 0x03 : /* 43   { "ORL %s,#%s",2,1,3}, *
                             * 53   { "ANL %s,#%s",2,1,3}, *
                             * 63   { "XRL %s,#%s",2,1,3}, */
                  Data -> SetDDato (Opr,Oper (Opcode,Data -> GetDDato (Opr),
                    Code -> GetDato (Pc + 2)));
                  break;
                case 0x05 : /* 25   { "ADD A,%s",2,0,2}, *
                             * 35   { "ADDC A,%s",2,0,2}, *
                             * 45   { "ORL A,%s",2,0,2}, *
                             * 55   { "ANL A,%s",2,0,2}, *
                             * 65   { "XRL A,%s",2,0,2}, *
                             * 95   { "SUBB A,%s",2,0,2}, */
                  Data -> SetDDato (aACC,Oper (Opcode,Data -> GetDDato (aACC ),
                    Data -> GetDDato (Opr)));
                  break;
              }
              break;
          }
          break;
      };
      break;
    case 3 : //   Bit
      Opr = Code -> GetDato (Pc + 1);
      switch (Opcode)
      {
        case 0x10 : /*  { "JBC %s,%s",3,5,3}, */
          DoJmp = Data -> GetBit (Opr);
          Data -> SetBit (Opr,0);
          if (DoJmp)
            return GetArgVal (2,Pc,Args[1]);
          break;
        case 0x20 : /*  { "JB %s,%s",3,5,3}, */
          if (Data -> GetBit (Opr))
            return GetArgVal (2,Pc,Args[1]);
           break;
        case 0x30 : /*  { "JNB %s,%s",3,5,3}, */
          if (! Data -> GetBit (Opr))
            return GetArgVal (2,Pc,Args[1]);
          break;
        case 0x72 : /*  { "ORL C,%s",3,0,2}, */
          Data -> SetBit (bCY,Data -> GetBit (bCY) || Data -> GetBit (Opr));
          break;
        case 0x82 : /*  { "ANL C,%s",3,0,2}, */
          Data -> SetBit (bCY,Data -> GetBit (bCY) && Data -> GetBit (Opr));
          break;
        case 0x92 : /*  { "MOV %s,C",3,0,2}, */
          Data -> SetBit (Opr,Data -> GetBit (bCY));
          break;
        case 0xA0 : /*  { "ORL C,/%s",3,0,2}, */
          Data -> SetBit (bCY,Data -> GetBit (bCY) || !Data -> GetBit (Opr));
          break;
        case 0xA2 : /*  { "MOV C,%s",3,0,2}, */
          Data -> SetBit (bCY,Data -> GetBit (Opr));
          break;
        case 0xB0 : /*  { "ANL C,/%s",3,0,2}, */
          Data -> SetBit (bCY,Data -> GetBit (bCY) && !Data -> GetBit (Opr));
          break;
        case 0xB2 : /*  { "CPL %s",3,0,2}, */
          Data -> SetBit (Opr,!Data -> GetBit (Opr));
          break;
        case 0xC2 : /*  { "CLR %s",3,0,2}, */
          Data -> SetBit (Opr,0);
          break;
        case 0xD2 : /*  { "SETB %s",3,0,2}, */
          Data -> SetBit (Opr,1);
          break;
      }
      break;
    case 4 : //    MOV DPTR,#num
      Data -> SetDDato (aDPH,Code -> GetDato (Pc + 1));
      Data -> SetDDato (aDPL,Code -> GetDato (Pc + 2));
      break;
    case 5 : //    <jmp> <rel>
      switch (Opcode)
      {
        case 0x40 :     /*      { "JC %s",5,0,2}, */
          DoJmp = Data -> GetBit (bCY);
          break;
        case 0x50 : /*  { "JNC %s",5,0,2}, */
          DoJmp = !Data -> GetBit (bCY);
          break;
        case 0x60 : /*  { "JZ %s",5,0,2}, */
          DoJmp = !Data -> GetDDato (aACC);
          break;
        case 0x70 : /*  { "JNZ %s",5,0,2}, */
          DoJmp = Data -> GetDDato (aACC);
          break;
        case 0x80 : /*  { "SJMP %s",5,0,2}, */
          DoJmp = 1;
          break;
        default : /* D8.DF{ "DJNZ RX,%s",5,0,2}, */
          DoJmp = Data -> GetRegister (Opcode) - 1;
          Data -> SetRegister (Opcode,DoJmp);
          break;
      }
      if (DoJmp) return GetArgVal (1,Pc,Args [0]);
      break;
    case 6 :
      switch (Opcode)
      {
        case 0x32 :     //      { "RETI",6,0,1},
          /* <<< Aggiungere gestione Int Flags >>> */
        case 0x22 :     //      { "RET",6,0,1},
          DoJmp = Data -> Pop () << 8;
          DoJmp |= (Data -> Pop () & 0xff);
          return DoJmp;
        case 0x73 :     //      { "JMP @A+DPTR",6,0,1},
          DoJmp = ((Data -> GetDDato (aDPH) << 8) & 0xff00) |
             (Data -> GetDDato (aDPL) & 0xff);
          DoJmp += Data -> GetDDato (aACC) & 0xff;
          return DoJmp;
      }
      break;
    case 7 : //    LCALL/LJMP
    case 0x80 : //..0x87 (AJMP/ACALL) 0..7 -> bit 10..8, byte seguente -> byte 7..0
    case 0x81 :
    case 0x82 :
    case 0x83 :
    case 0x84 :
    case 0x85 :
    case 0x86 :
    case 0x87 :
      if (Opcode & 0x10)
      {       // Call -> push PC
        DoJmp = Pc + Len;
        Data -> Push (DoJmp);
        Data -> Push (DoJmp >> 8);
      }
      return GetArgVal (1,Pc,Args [0]);
  }
  return Pc + Len;        // Ritorna il nuovo PC
}

unsigned OpCode::GetMnemonic (unsigned Ad,char *s)
{
char A1 [10],A2 [10];
	MkArg (A1,1,Ad,Args [0]);
	MkArg (A2,2,Ad,Args [1]);
	sprintf (s,Mnemonic,A1,A2);
	return Len;
}

int OpCode::Asm (int num,argStore *foundPars,unsigned Ad, unsigned char *a, bool secondpass)
{
int i, arg=0;
int offs;
	for (i = 0;i < 3; i++) {	// Confronta i parametri attuali con quelli "attesi"
		if (pars[i] != foundPars[i].type && (foundPars[i].type!=dir || pars[i]!=bitDir)) {
			return 0;
		}
		switch (pars[i]) {
			case Pimm:
				if (Args [arg] == imm16) {	// Mov DPTR,#imm16
					a [++arg] = foundPars[i].val >> 8;
					a [++arg] = foundPars[i].val;
				} else {
					if ((foundPars[i].val > 255 || foundPars[i].val < -128) && secondpass) {
						ErrCode = toobig;
					}
					a [++arg] = foundPars[i].val;
				}
				break;
			case bitDir:
			case notBitDir:
				if ((foundPars[i].val > 255 || foundPars[i].val < 0) && secondpass) {
					ErrCode = toobig;
				}
				a [++arg] = foundPars[i].val;
				break;
			case dir:
				switch (Args [arg]) {
					case data:
						if ((foundPars[i].val > 255 || foundPars[i].val < 0) && secondpass) {
							ErrCode = toobig;
						}
						a [++arg] = foundPars[i].val;
						break;
					case imm16:
					case absaddr:
						a [++arg] = foundPars[i].val >> 8;
						a [++arg] = foundPars[i].val;
						break;
					case reladdr:
						Ad += Len;
						// Controlla limiti solo nella seconda passata
						offs = (foundPars[i].val - Ad);
						if ((offs < - 128 || offs > + 127) && secondpass) {
							ErrCode = toofar;
						}
						a [++arg] = offs;
						break;
					default:	// (noarg, imm, bit, fixedarg - questi non arrivano) paddr80 paddr81 paddr82
								// paddr83 paddr84 paddr85 paddr86 paddr87
						// Controlla i limiti solo nella seconda passata
						if ((unsigned)(foundPars[i].val & 0xf800) != (unsigned)(Ad & 0xf800) && secondpass) {
							ErrCode = toofar;
						}
						// crea l'indirizzo all'interno del banco
						num = (num & 0x1f) | (((unsigned)foundPars[i].val >> 3) & 0xe0);
						a [++arg] = foundPars[i].val;
						break;
				}
				break;
			case reg:
			case indReg:
				num |= foundPars[i].val;
				break;
			default:	// indDptr, idxPc, idxDptr, dptrReg, aReg, cReg, bReg
				break;
		}
	}
	a [0] =  num;
	return Len;
}

/*
// Serve per impostare il breakpoint del single step !!!
unsigned OpCode::NextIst (unsigned char *p,unsigned Ad,int Calls)
{
int i;
unsigned T,T1,T2;
  i = (Args [1] >= 5) ? Args [1] : Args [0];
  T = Ad + Len;
  switch (i)
  {
    case 5 : //    <jmp> <rel>
      T1 = Ad + ((char) p [1]) + Len;
      T2 = Ad + ((char) p [2]) + Len;
      switch (p [0])
      {
        case 0x10 :     //      { "JBC %s,%s",3,5,3},
        case 0x20 :     //      { "JB %s,%s",3,5,3},
          if (Data -> GetBit (p [1]))
            T = T2;
          break;
        case 0x30 :     //      { "JNB %s,%s",3,5,3},
          if (!Data -> GetBit (p [1])) 
            T = T2;
          break;
        case 0x40 :     //      { "JC %s",5,0,2},
          if (Data -> GetBit (bCY)) 
            T = T1;
          break;
        case 0x50 :     //      { "JNC %s",5,0,2},
          if (!Data -> GetBit (bCY)) 
            T = T1;
          break;
        case 0x60 :     //      { "JZ %s",5,0,2},
          if (!Data -> GetDDato (aACC)) 
            T = T1;
          break;
        case 0x70 :     //      { "JNZ %s",5,0,2},
          if (Data -> GetDDato (aACC)) 
            T = T1;
          break;
        case 0x80 :     //      { "SJMP %s",5,0,2},
          T = T1;
          break;
        case 0xB4 :     //      { "CJNE A,#%s,%s",1,5,3},
          if (Data -> GetDDato (aACC) != p [1]) 
            T = T2;
          break;
        case 0xB5 :     //      { "CJNE A,%s,%s",2,5,3},
          if (Data -> GetDDato (aACC) != Data -> GetDato (p [1])) 
            T = T2;
          break;
        case 0xB6 :     //      { "CJNE @R0,#%s,%s",1,5,3},
        case 0xB7 :     //      { "CJNE @R1,#%s,%s",1,5,3},
          if (Data -> GetDato (Data -> GetRegister (p [0] & 1)) != p [1]) 
            T = T2;
          break;
        case 0xB8 :     //      { "CJNE R0,#%s,%s",1,5,3},
        case 0xB9 :     //      { "CJNE R1,#%s,%s",1,5,3},
        case 0xBA :     //      { "CJNE R2,#%s,%s",1,5,3},
        case 0xBB :     //      { "CJNE R3,#%s,%s",1,5,3},
        case 0xBC :     //      { "CJNE R4,#%s,%s",1,5,3},
        case 0xBD :     //      { "CJNE R5,#%s,%s",1,5,3},
        case 0xBE :     //      { "CJNE R6,#%s,%s",1,5,3},
        case 0xBF :     //      { "CJNE R7,#%s,%s",1,5,3},
          if (Data -> GetRegister (p [0] & 7) != p [1]) 
            T = T2;
          break;
        case 0xD5 :     //      { "DJNZ %s,%s",2,5,3},
          if (Data -> GetDato (p [1]) != 1) 
            T = T2;
          break;
        case 0xD8 :     //      { "DJNZ R0,%s",5,0,2},
        case 0xD9 :     //      { "DJNZ R1,%s",5,0,2},
        case 0xDA :     //      { "DJNZ R2,%s",5,0,2},
        case 0xDB :     //      { "DJNZ R3,%s",5,0,2},
        case 0xDC :     //      { "DJNZ R4,%s",5,0,2},
        case 0xDD :     //      { "DJNZ R5,%s",5,0,2},
        case 0xDE :     //      { "DJNZ R6,%s",5,0,2},
        case 0xDF :     //      { "DJNZ R7,%s",5,0,2},
          if (Data -> GetRegister (p [0] & 0x7) != 1) 
            T = T1;
          break;
      }
      break;
    case 6 : //    JMP @A+DPTR,RET,RETI
      switch (p [0])
      {
        case 0x22 :     //      { "RET",6,0,1},
        case 0x32 :     //      { "RETI",6,0,1},
          T = Data -> GetTopOfStack ();
          break;
        case 0x73 :     //      { "JMP @A+DPTR",6,0,1},
          T = ((Data -> GetDDato(aDPH) << 8) & 0xff00) |
            (Data -> GetDDato(aDPL) & 0xff);
          T += Data -> GetDDato (aACC);
          break;
      }
      break;
    case 7 : //    LCALL/LJMP
      T1 = (((unsigned) p [1]) << 8) | ((unsigned) p [2]);
      if (p [0] != 0x12 || Calls) 
        T = T1;
      break;
    case 0x80 : //..0x87 (AJMP/ACALL) 0..7 -> bit 10..8, byte seguente -> byte 7..0
    case 0x81 :
    case 0x82 :
    case 0x83 :
    case 0x84 :
    case 0x85 :
    case 0x86 :
    case 0x87 :
      T1 = (Ad & 0xf800) | ((i << 8) & 0x700) | p [1];
      if ((p [0] & 0x1f) != 0x11 || Calls)
        T = T1;
      break;
  }
  return (T < Ad - 3 || T > Ad ) ? T : Ad + Len;
}
 */

void OpCode::add (parseNode *&parseTree, unsigned char opc)
{
char *s;
	if (Mnemonic [0] < 'A' || Mnemonic [0] >= 'z') {	// Se non comincia per carattere
		return;	// non aggiunge
	}
	if ((s = strchr (Mnemonic, ' '))) {	// Se ha parametri
		if ((s = strchr (s, 'R')))	{ 	// c'e' un parametro che inizia per "R"
			if (s [1] > '0' && s [1] < '8') {	// e risulta essere un registro diverso da R0
				return;
			}
		}
	}
	if (!parseTree) {
		parseTree = new parseNode (Mnemonic [0], NULL);
	}
	parseNode::addToRoot (parseTree, Mnemonic, opcode, opc);
}

/*
 * Node
 */

Node *Node::add (Node **parseTree, char *str)
{
	if (toupper (*str) == c) {	// Carattere giusto
		if (issym_cont(str [1])) {	// Stringa non finita (ancora lettere)
			if (!nextLevel) {
				nextLevel = New (str [1], NULL);
			}
			return nextLevel -> add (&nextLevel, str + 1);
		}
	} else	if (toupper (*str) < c) {	// Deve aggiungere nuovo nodo prima
		*parseTree =  New (*str, this);
		return (*parseTree) -> add (parseTree, str);
	} else {	// Prova con il prossimo nodo
		if (!nextChar) {
			nextChar = New (*str, NULL);
		}
		return nextChar -> add (&nextChar, str);
	}
	return this;
}

Node *Node::find (char *str)
{
	if (toupper  (*str) == c) {	// Carattere corrisponde
		if (issym_cont (str [1])) {	// La stringa continua
			if (nextLevel) {
				return nextLevel -> find (str + 1);
			}
		} else {	// Stringa trovata
			return this;
		}
	} else if (toupper  (*str) > c) {	// Sono ordinati...
		if (nextChar) {
			return nextChar -> find (str);
		}
	}
	return NULL;
}

/*
 * opcContent
 */
/* Sorta gli opcode mettendo quelli più corti prima */
void opcContent::add (unsigned char code, nodeType t)
{
	type = t;
	if (t == opcode) {
	int i,len = i8052::IstSet [code].GetLen();
		for (i = nopcode; i && i8052::IstSet [opcodeList [i-1]].GetLen() > len; i--) {
			opcodeList [i] = opcodeList [i-1];	// Sposta avanti quelli maggiori
		}
		opcodeList [i] = code;
		nopcode ++;
	} 
}

/*
 *	parseNode
 */
int parseNode::maxopcode = 0;
int parseNode::nnodes = 0;

void parseNode::setContent (nodeType type, unsigned char opc)
{
	if (content.getNum ()  > maxopcode) {
		maxopcode = content.getNum ();
	}
	content.add (opc, type);
}

void parseNode::addToRoot (parseNode *&parseTree, char *str, nodeType type, unsigned char opc)
{
	if (!parseTree) {
		parseTree =  new parseNode (*str, NULL);
	}
	parseNode *n = (parseNode *) parseTree -> add ((Node **)&parseTree, str);
	n -> setContent (type, opc);
}

opcContent *parseNode::find (char *str)
{
parseNode *f = (parseNode *) Node::find (str);
	if (f) {	// Carattere corrisponde
		return & f -> content;
	}
	return NULL;
}

/*
 * symtabNode
 */
bool symtabNode::addToRoot (symtabNode *&symTab, char *str, int v)
{
	if (!symTab) {
		symTab =  new symtabNode (*str, NULL);
	}
	symtabNode *n = (symtabNode *) symTab -> add ((Node **)&symTab, str);
	return n -> content.set (v);
}

symContent *symtabNode::find (char *str)
{
symtabNode *f = (symtabNode *) Node::find (str);
	if (f) {	// Carattere corrisponde
		return & f -> content;
	}
	return NULL;
}
