/***************************************************************************
 *  pc.cc
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 * Pc e breackpoints (3 su 4, il quarto per 'go to line') 
 *  Attenzione!!
 *	Nel byte dove e' posto un breackpoint e nei due successivi non possono
 *	esserci ne' un altro breakpoint ne il PC.
 *	Per la posizione di un breakpoint si puo' passare in single step (vedere che farne)
 *  Controllare, quando si lancia il programma, che nessun breackpoint coincida con il PC
 */
 
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include <glib/gmacros.h>
#include <glib.h>
#include <gtkmm.h>
#include <libgnomemm.h>

#include <iostream>	// Per la stampa dell'eventuale errore

#include "pcmon.h"
#include "sfr.h"
#include "dialogs.h"

#ifndef _MESSAGEINTERFACE_H
#include "messageinterface.h"
#endif

#ifndef _DMEMORY_H
#include "dmemory.h"
#endif

#ifndef _I8052_H
#include "i8052.h"
#endif
#include "pc.h"

void Pc::toolbarBuild (Pcmon *parent)
{
  //Create actions for menus and toolbars:
  m_refActionGroup = Gtk::ActionGroup::create();

  m_refActionGroup->add( Gtk::Action::create("Esegui", Gtk::Stock::EXECUTE),
  	sigc::mem_fun(*parent,&Pcmon::menuRun) );
  m_refActionGroup->add( Gtk::Action::create("EseguiInizio", Gtk::StockID("pcmon_btn_runstart")),
  	sigc::mem_fun(*parent,&Pcmon::menuRunStart) );
  m_refActionGroup->add( Gtk::Action::create("PassoSingolo", Gtk::StockID("pcmon_btn_singlestep")),
  	sigc::mem_fun(*parent,&Pcmon::menuStep) );

  m_refActionGroup->add( Gtk::Action::create("Ricarica", Gtk::Stock::REFRESH),
  	sigc::mem_fun(*this,&Pc::menuRefresh) );

  m_refUIManager = Gtk::UIManager::create();
  m_refUIManager->insert_action_group(m_refActionGroup);
  add_accel_group(m_refUIManager->get_accel_group());

  try
  {
    /* Crea l'immagine per tutti i menu (principale e popup) e le toolbar */
    Glib::ustring ui_info =
        "<ui>"
        "  <toolbar  name='ToolBar'>"
        "    <toolitem action='Esegui'/>"
        "    <toolitem action='EseguiInizio'/>"
        "    <toolitem action='PassoSingolo'/>"
        "    <separator/>"
		"    <toolitem action='Ricarica'/>"
        "  </toolbar>"
        "</ui>";

    m_refUIManager->add_ui_from_string(ui_info);
  }
  catch(const Glib::Error& ex)
  {
    std::cerr << "Preparazione dei menu fallita: " <<  ex.what();
  }

  //Get the toolbar widgets, and add them to a container widget:
  Gtk::Toolbar* toolbar = (Gtk::Toolbar *) m_refUIManager->get_widget("/ToolBar") ;
  if(toolbar)
    vbox.pack_start(*toolbar, Gtk::PACK_SHRINK);
  toolbar->set_toolbar_style(Gtk::TOOLBAR_ICONS);
}

void Pc::redraw ()
{
	pc.redraw ();
	b1.redraw ();
	b2.redraw ();
	b3.redraw ();
}

Pc::Pc (Pcmon *p, i8052 *b) :
  pc ("Pc", false, false, b -> GetSFR ()->GetPcReg()),
  b1 ("Breackpoint 1", false, true, b -> GetSFR ()->GetB1Reg()),
  b2 ("Breackpoint 2", false, true, b -> GetSFR ()->GetB2Reg()),
  b3 ("Breackpoint 3", false, true, b -> GetSFR ()->GetB3Reg())
{
	m = b->GetData ();

	set_resizable (false);	// Finestra non ridimensionabile
	set_title ("Pc e Breakpoint");

	toolbarBuild (p);

	vbox.pack_start(*pc.getBox (), false,false,0);
	vbox.pack_start(*b1.getBox (), false,false,0);
	vbox.pack_start(*b2.getBox (), false,false,0);
	vbox.pack_start(*b3.getBox (), false,false,0);

	add (vbox);
	redraw ();
	show_all ();
}
