/***************************************************************************
 *  ppi.cc
 *  Emulatore di 8255 connesso a dip switch e TIO16
 *  Conteienanche i due LED di satato (P1.2 e P1.5)
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include <glib/gmacros.h>
#include <glib.h>
#include <gtkmm.h>
#include <libgnomemm.h>

#include "ppi.h"

#include "../pixmaps/green_led_on.xpm"
#include "../pixmaps/green_led_off.xpm"
#include "../pixmaps/red_led_on.xpm"
#include "../pixmaps/red_led_off.xpm"
#include "../pixmaps/yellow_led_on.xpm"
#include "../pixmaps/yellow_led_off.xpm"
#include "../pixmaps/red_button_open.xpm"
#include "../pixmaps/red_button_closed.xpm"
#include "../pixmaps/green_button_open.xpm"
#include "../pixmaps/green_button_closed.xpm"
#include "../pixmaps/yellow_button_open.xpm"
#include "../pixmaps/yellow_button_closed.xpm"
#include "../pixmaps/dip_open.xpm"
#include "../pixmaps/dip_closed.xpm"

const bobject ppi::inits [nobjects] = {
  { 10, 10, 0x1, portA, redled, led},
  { 10, 27, 0x2, portA, redled, led},
  { 10, 44, 0x4, portA, redled, led},
  { 10, 61, 0x8, portA, redled, led},
  { 10, 78, 0x10, portA, redled, led},
  { 10, 95, 0x20, portA, redled, led},
  { 10, 112, 0x40, portA, redled, led},
  { 10, 129, 0x80, portA, redled, led},

  { 10, 146, 0x1, portC, yellowled, led},
  { 10, 163, 0x2, portC, yellowled, led},
  { 10, 180, 0x4, portC, yellowled, led},
  { 10, 197, 0x8, portC, yellowled, led},
  { 10, 214, 0x10, portC, greenled, led},
  { 10, 231, 0x20, portC, greenled, led},
  { 10, 248, 0x40, portC, greenled, led},
  { 10, 265, 0x80, portC, greenled, led},

  { 29, 32, 0x1, inportA, redbutton, button},
  { 29, 62, 0x2, inportA, redbutton, button},
  { 29, 92, 0x4, inportA, redbutton, button},
  { 29, 122, 0x8, inportA, redbutton, button},
  { 29, 152, 0x10, inportA, redbutton, button},
  { 29, 182, 0x20, inportA, redbutton, button},
  { 29, 212, 0x40, inportA, redbutton, button},
  { 29, 242, 0x80, inportA, redbutton, button},

  { 59, 32, 0x1, inportC, yellowbutton, button},
  { 59, 62, 0x2, inportC, yellowbutton, button},
  { 59, 92, 0x4, inportC, yellowbutton, button},
  { 59, 122, 0x8, inportC, yellowbutton, button},
  { 59, 152, 0x10, inportC, greenbutton, button},
  { 59, 182, 0x20, inportC, greenbutton, button},
  { 59, 212, 0x40, inportC, greenbutton, button},
  { 59, 242, 0x80, inportC, greenbutton, button},

  { 119, 97, 0x1, portB, dipswitch, dip},
  { 119, 107, 0x2, portB, dipswitch, dip},
  { 119, 117, 0x4, portB, dipswitch, dip},
  { 119, 127, 0x8, portB, dipswitch, dip},
  { 119, 137, 0x10, portB, dipswitch, dip},
  { 119, 147, 0x20, portB, dipswitch, dip},
  { 119, 157, 0x40, portB, dipswitch, dip},
  { 119, 167, 0x80, portB, dipswitch, dip},

  { 120, 36, 0x4, P1, yellowled, led},
  { 120, 53, 0x20, P1, greenled, led},
};

binary::binary (int const X, int const Y, Glib::RefPtr<Gdk::Pixmap> Off, Glib::RefPtr<Gdk::Pixmap> On, int *Port, int const Mask)
{
	x = X; 
	y = Y; 
	off = Off; 
	on = On; 
	port = Port; 
	mask = Mask; 
    off -> get_size (width, height); 
}

void binary::show (Glib::RefPtr<Gdk::Pixmap> win, Glib::RefPtr<Gdk::GC> gc)
{
	win->draw_drawable (gc, (*port & mask) ? off : on, 0, 0, x, y);
}

void binary::redraw (ppi *parent)
{
  parent -> redraw (x, y, width, height);
}

void Button::click (int press, int xb, int yb, ppi *parent)
{
  if (xb > x && yb > y && xb < x + width && yb < y + height)
  {	// Click nell'area
    if (!press) *port |= mask;
    else *port &= ~mask;
    parent -> redraw (x, y, width, height); 
  }
}

void Dipsw::click (int press, int xb, int yb, ppi *parent)
{
  if (xb > x && yb > y && xb < x + width && yb < y + height)
  {	// Click nell'area
    if (press)
      *port ^= mask;
    parent -> redraw (x, y, width, height); 
  }
}

void ppi::redrawPorts (int emulating)
{
  if (outports [3] &0x1)	// Port C low = out
    ports [portC] = (ports [portC] & 0xf0) | (ports [inportC] & 0x0f);
  else
    ports [portC] = (ports [portC] & 0xf0) | (outports [portC] & ports [inportC] & 0x0f);
  if (outports [3] &0x8)	// Port C hi = out
    ports [portC] = (ports [portC] & 0x0f) | (ports [inportC] & 0xf0);
  else
    ports [portC] = (ports [portC] & 0x0f) | (outports [portC] & ports [inportC] & 0xf0);
  if (outports [3] &0x10)	// Port C hi = out
    ports [portA] = ports [inportA];
  else
    ports [portA] = outports [portA] & ports [inportA];

  if (emulating) {
    toRedraw = true;
  } else {
    redraw ();
  }
}

void ppi::Out (int address, int datum, int emulating)
{ 
	address &= 3;
	if (address < 3 || (datum & 0x80)) {	// Scarta controllo input
		outports [address] = datum;
	}
	redrawPorts (emulating);
}

void ppi::P1Out (int datum, int emulating)
{
	ports [4] = datum; 
	if (emulating) {
		toRedraw = true;
	} else { 
		disps [40] -> redraw (this); 
		disps [41] -> redraw (this); 
	} 
}

unsigned ppi::In (int address)
{
  address &= 3;
  return (address < 3) ? ports [address] : outports [3]; 
}

void ppi::redrawIfChanged()
{
	if (toRedraw) {
		toRedraw = false;
		redraw ();
	}
}

bool ppi::on_button_press_event (GdkEventButton *b)
{
int i;
  for (i = 0; i < nobjects; i ++)
  {
    disps [i] -> click (true, (int) (b -> x), (int) (b -> y), this);
  }
  redrawPorts (0);
  return true;
}

bool ppi::on_button_release_event (GdkEventButton *b)
{
int i;
  for (i = 0; i < nobjects; i ++)
  {
    disps [i] -> click (false, (int) (b -> x), int (b -> y), this);
  }
  redrawPorts (0);
  return true;
}

Glib::RefPtr<Gdk::Pixmap>ppi::makeIcon (char **icon)
{
Glib::RefPtr<Gdk::Bitmap> g_mask;
Glib::RefPtr<Gdk::Pixmap> g_pixmap;
	return Gdk::Pixmap::create_from_xpm  ( get_default_colormap(), g_mask,icon);
}

bool ppi::on_configure_event (GdkEventConfigure *event)
{
int i;
Glib::RefPtr<Gdk::GC> gc;
Glib::RefPtr<Gdk::GC> some_gc = Gdk::GC::create(get_window());
Gdk::Color c;   

  images [redled][0] = makeIcon (red_led_off_xpm);
  images [redled][1] = makeIcon (red_led_on_xpm);
  images [greenled][0] = makeIcon (green_led_off_xpm);
  images [greenled][1] = makeIcon (green_led_on_xpm);
  images [yellowled][0] = makeIcon (yellow_led_off_xpm);
  images [yellowled][1] = makeIcon (yellow_led_on_xpm);
  images [redbutton][0] = makeIcon (red_button_open_xpm);
  images [redbutton][1] = makeIcon (red_button_closed_xpm);
  images [greenbutton][0] = makeIcon (green_button_open_xpm);
  images [greenbutton][1] = makeIcon (green_button_closed_xpm);
  images [yellowbutton][0] = makeIcon (yellow_button_open_xpm);
  images [yellowbutton][1] = makeIcon (yellow_button_closed_xpm);
  images [dipswitch][0] = makeIcon (dip_open_xpm);
  images [dipswitch][1] = makeIcon (dip_closed_xpm);

  /* if (!pixmap.is_null ()) { non occorre piu' l' "is_null" */
  if (pixmap) {
    pixmap.clear ();
  }

  pixmap = Gdk::Pixmap::create (get_window (), get_width(), get_height(),-1);
 
  gc = get_style()->get_white_gc();
  pixmap -> draw_rectangle (gc, true, 0, 0, get_width(), get_height());

  // Banco dipswitch (blu)
  Gdk::Color some_color;
  Glib::RefPtr<Gdk::Colormap> some_colormap = get_default_colormap();
  some_color.set_red(0x2000);
  some_color.set_green(0x2000);
  some_color.set_blue(0xf000);
  some_colormap->alloc_color(some_color);
  some_gc->set_foreground(some_color);
  pixmap -> draw_rectangle (some_gc, true, 87, 110, 95, 30);

  // Stampato (verde)
  some_color.set_red(0x3000);
  some_color.set_green(0xC000);
  some_color.set_blue(0x3000);
  some_colormap->alloc_color(some_color);
  some_gc->set_foreground(some_color);
  pixmap -> draw_rectangle (some_gc, true, 0, 0, 295, 100);

  // Posiziona i componenti dell'interfaccia
  for (i = 0; i < nobjects; i++)
    switch (inits [i].how)
    {
      case led:
		disps [i] = new binary (inits [i].x, inits [i].y, images [inits [i].what][0], 
		  images [inits [i].what][1], ports + inits [i].command, inits [i].mask);
		break;
      case button:
		disps [i] = new Button (inits [i].x, inits [i].y, images [inits [i].what][0], 
	  		images [inits [i].what][1], ports + inits [i].command, inits [i].mask);
		break;
      case dip:
		disps [i] = new Dipsw (inits [i].x, inits [i].y, images [inits [i].what][0], 
	  		images [inits [i].what][1], ports + inits [i].command, inits [i].mask);
		break;
    }

    return true;
}

bool ppi::on_expose_event (GdkEventExpose *event)
{
int i;
Glib::RefPtr<Gdk::GC> gc = get_style()->get_white_gc();
Glib::RefPtr<Gdk::Window> win = get_window();

	for (i = 0; i < nobjects; i ++)
    {
    	disps [i] -> show (pixmap, gc);
    }
    win -> draw_drawable(gc,
		    pixmap,
		    event->area.x, event->area.y,
		    event->area.x, event->area.y,
		    event->area.width, event->area.height);

    return false;
}

void ppi::redraw (int x, int y, int w, int h)
{
  queue_draw_area (x,y, (w > 0) ? w : Width, (h > 0) ? h : Height);
  toRedraw = false;
}

ppi::ppi ()
{
  toRedraw = false;
  /* pixmap = 0; Questo e' inutile perche' refptr si inizializza da solo... */

  ports [0] = 0xff;
  ports [1] = 0xff;
  ports [2] = 0xff;
  ports [3] = 0xaa;
  ports [5] = 0xff;
  ports [6] = 0xff;
  ports [7] = 0xff;
  outports [0] = 0xff;
  outports [1] = 0xff;
  outports [2] = 0xff;

  set_events (Gdk::EXPOSURE_MASK | Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK);
}
