/***************************************************************************
 *  serial.cc
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdio.h>
#include <sched.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <fcntl.h>
#include <string.h>
 
#include <glib/gmacros.h>
#include <glib.h>
#include <gtkmm.h>
#include <libgnomemm.h>
 
#include "serial.h"

// #define ERRORPRINT(...) printf (__VA_ARGS__)
#define ERRORPRINT(...)

Serial *Serial_Container::s = NULL;

// Stringhe da riconoscere per terminare lo strato di 'running'
static char sReset [] =
"\n\rMO52 Version 1.1\n\r\nGrifo(r) ITALIAN TECHNOLOGY\n\rTel.+39 051 892 052      Fax.+39 051 893 661";
 // "\n\rhttp://www.grifo.it\n";
static char sBreak [] = "Break at ";

void Serial::schPuts (const char *s)
{
	schGet ();
	if (com) {
		write (com, s, strlen (s));
		ERRORPRINT("%s", s);
	}
}

void Serial::schPutc (const char c)
{
	if (!interface->isRunning ())
	{
		schGet ();
		if (com) {
			write (com, &c, 1);
			ERRORPRINT("%c", c);
		}
	}
}

void Serial::schSendc (const char c)
{
	if (com) {
		write (com,&c,1);
	}
}

void Serial::schSends (const char *c)
{
	if (com) {
		write (com, c, strlen (c));
		ERRORPRINT("%s", c);
	}
}

/* Ritorna un carattere dalla pipe della scheda
 * con un timeout di 3 secondi
 * Se il carattere non arriva, ritorna -1;
 */
#define SCHWAIT	3
int Serial::schGetc ()
{
fd_set set;
struct timeval timeout;
unsigned char c;

	if (connected)
	{    
		// Initialize the file descriptor set.
		FD_ZERO (&set);
		FD_SET (boardPipe [0], &set);
     
		// Initialize the timeout data structure.
		timeout.tv_sec = SCHWAIT;
		timeout.tv_usec = 0;
		// `select' returns 0 if timeout, 1 if input available, -1 if error.
		if (!select (FD_SETSIZE, &set, NULL, NULL, &timeout)) {
			schPutc (0x1b);	// Se si e' interrotto per errore, interrompe il comando
			connected = false;
			interface -> setConnectedMessage ("Non Connesso");
			usePipe = false;
			return -1;
		}
		read (boardPipe [0],&c,1);
		ERRORPRINT("[%c]", c);
		return c;
	}
	return -1;
}

int Serial::schGets (char *str,int max)
{
int rec;
	do {
		rec = schGetc ();
		if (rec != '\n' && rec != '\r') {
			*str++ = rec;
		}
	} while (rec >= 0 && rec != '\r' && --max);
	*str = 0;
	return rec < 0;
}

int Serial::schWaitc (char c)
{
int rec;
	do {
		rec = schGetc ();
	} while (rec >= 0 && rec != c);
	return rec < 0;
}

void *Serial::receiver_wrapper (void *th)
{
    return ((Serial *)th) -> receiver ();
}

/* Secondo thread del programma;
 * Apre la seriale in ingresso ed attende i caratteri in ingresso, copiandoli
 * sul 'terminale virtuale'
 * Utilizza il suo file privato ed interno per la seriale*/
void *Serial::receiver ()
{
char c;
	while (true)
	{
		if (com) {
			read (com,&c,1);
		} else {
			return 0;
		}
		ERRORPRINT("<%c>", c);

		if (!connected)
		{
			interface -> setConnectedMessage ("Connesso");
			connected = true;
		}

		if (usePipe) {
			write (boardPipe [1],&c,1);
		} else {
			if (c == sReset [0] || c == sBreak [0]) {	// Possibile prompt, quindi rientro nel monitor
			fd_set set;
			struct timeval timeout;
			char *o = (c == sReset [0]) ? sReset : sBreak;
			char *s = o + 1;
				while (1) { /* Initialize the file descriptor set. */
					FD_ZERO (&set);
					FD_SET (com, &set);

					/* Initialize the timeout data structure. */
					timeout.tv_sec = 0;
					timeout.tv_usec = 100000;
					/* `select' returns 0 if timeout, 1 if input available, -1 if error. */
					if (select (FD_SETSIZE, &set, NULL, NULL, &timeout)) {
						read (com,&c,1);
						if (c != *s) {	// Stringa non trovata
							break;
						}
						if (!*++s) {	// Stringa trovata
							break;
						}
					} else {	// Carattere non arrivato
						c = 0;
						break;
					}
				}
				if (*s) {	// Stringa non corrisponde
					while (o < s) {
						interface -> tPutc (*o++);	// Stampa i caratteri saltati
					}
					if (c) {
						interface -> tPutc (c);	// Stampa il carattere non corrispondente
					}
				} else {	// Stringa trovata
					do {	// Attende il prompt '*'
						read (com,&c,1);
					} while (c != '*');
					if (interface->isRunning ())
					{
					int oldpipe;
						interface -> setRunningMessage ("Non In Esecuzione");
						interface->setRunning (false);
						oldpipe = boardPipe [0];
						boardPipe [0] = com;	// Va direttamente al file, saltando la pipe
						interface -> reload ();
						boardPipe [0] = oldpipe;
					}
				}
			} else {
				interface -> tPutc (c);
			}
		}
	}
	return NULL;
}

void Serial::noline ()
{
	setConnected (false);
	interface -> setConnectedMessage ("Non Connesso");
	usePipe = false;
	com = 0;
}

Serial::Serial (char *device, Serial_Container *p)
{
struct termios tattr;
int err;
	interface = p;
    comname = device;
	receiver_thread = 0;	// Inizialmente nessun thread

    usePipe = false;	// Input dalla scheda al terminale

    /* Se il nome file della seriale e' vuoto, non apro
     * la seriale e lavoro senza */
    if (comname [0] == 0)
    {
    	noline ();
		return;
    }

    if (pipe (boardPipe))
    {
		perror ("Non poso creare la pipe");
		exit (1);
    }

    com = open (comname,O_RDWR | O_SYNC);
    if (com < 0)
    {
		perror ("Non posso aprire la seriale"); 
    	noline ();
    } else {
		if (! isatty (com)) {
			// Non è un terminale - seriale inesistente
			fprintf (stderr,"Dispositivo %s: ",comname);
			perror ("Il file non e' una seriale"); 
			close (com);
    		noline ();
			return;
		}
		/* Save the terminal attributes so we can restore them later. */
		tcgetattr (com, &saved_attributes);
	
		/* Set the funny terminal modes. */
		tcgetattr (com, &tattr);
		tattr.c_iflag &= ~(INLCR|IGNCR|ICRNL|IXON); /*  */
		tattr.c_oflag &= ~(OPOST|ONLCR|OCRNL|ONLRET); /*  */
		tattr.c_cflag = CS8 | CREAD | CLOCAL; 
		tattr.c_lflag &= ~(ICANON|ECHO); /* Clear ICANON and ECHO. */
		tattr.c_cc[VMIN] = 1;
		tattr.c_cc[VTIME] = 0;
		cfsetispeed (&tattr, B9600);
		cfsetospeed (&tattr, B9600);
		tcsetattr (com, TCSAFLUSH, &tattr);
	}

    err = pthread_create (&receiver_thread, NULL, receiver_wrapper, this);
    if (err)
    {
		printf ("Non posso creare il thread del terminale (%d) \n",err);
		exit (1);
    }
}

Serial::~Serial ()
{
	if (receiver_thread) {
		pthread_cancel (receiver_thread);
	}
	if (com) {
		tcsetattr (com, TCSANOW, &saved_attributes);
		close (com);
	}
}
