/***************************************************************************
 *  xmemory.cc
 *
 *  Sun Nov 14 10:49:20 2004
 *  Copyright  2004 Ing. Stefano Salvi
 *  Email stefano@salvi.mn.it
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/* Gal memoria: BS
 * J2 2-3 -> Ram IC8 32K
 * J3 2-3 -> Rom IC7 32K
 * J4 1-2
 * J5 2-3
 * Mappaggio memoria: BS-1
 *  Eprom: 0000-7fff
 *  RAM:   8000-f7ff
 *  I/O:   f800-ffff
 *	PPI 8255:    fa00
 *	Tim a 8254:  fa40
 *	Tim b 8254:  fa80
 *	RTC:	     fac0
 *	Bus:	     fb00
 */

#include <stdio.h>
#include "xmemory.h"
#include "serial.h"
#ifndef _PPIINTERFACE_H
#include "ppiinterface.h"
#endif

/*
 * XDataMemory
 */
extern ppiinterface DummyPPI;

XDataMemory::XDataMemory (int c_start, int c_size) :
  Memory (c_start, c_size)
{
	masks = new unsigned char [c_size / 8];
	PPI = &DummyPPI;
}

void XDataMemory::SetDato (unsigned ad,char val)
{
	if (ad >= 0xfa00)
	{
		if (ad < 0xfa40) {
			PPI -> Out (ad, val, status == emulating);
		}
	} 
	else 
	{
		ad -= start;
		if (ad < (unsigned)size) {
			data [ad] = val & 0xff; 
			if (status != emulating) {
				redraw ();
			}
		}
	}
}

unsigned char XDataMemory::GetDato (unsigned ad)
{ 
  if (ad >= 0xfa00)
  {
		if (ad < 0xfa40) {
			return PPI -> In (ad);
		}
  } 
  ad -= start;
  return (ad < (unsigned)size) ? data [ad] : 0;
}

void XDataMemory::checkRange (int r_start, int r_end)
{
int ad;
int stad = 0xffff;
int endad = 0; 

    // Controlla se gli indirizzi visibili sono stati letti
    for (ad = r_start; ad < r_end; ad += 8)
    {
        if (masks [(ad - start) / 8] != 0xff)
		{
	    	if (stad > ad)
			{
				stad = ad;
			}
	    	if (endad < ad + 7)
			{
				endad = ad + 7;
			}
		}
    }
    // Se un'area non e' stata letta, la legge
    if (stad < endad)
	{
		load (stad, endad);
	}
}

void XDataMemory::load (int stad, int endad)
{
int i,j;
char inbuf [80];
int vals [16];

	// Se la scheda e' gia' in uso da un altro processo
	if (status != stopped) {
		return;
	}

	Serial_Container::s->schPutc ('D');
	if (Serial_Container::s->schWaitc (' ')) {
		return;
	}
	sprintf (inbuf,"%04X",stad);
	Serial_Container::s->schPuts (inbuf);
	sprintf (inbuf,"%04X",endad);
	if (Serial_Container::s->schWaitc (',')) {
		return;
	}
	Serial_Container::s->schPuts (inbuf);
	if (Serial_Container::s->schGets (inbuf,49)) {	// Risucchia linea di comando
		return;
	}
	for (i = stad; i < endad; i += 16)
	{
		if (Serial_Container::s -> schGets (inbuf,79)) {
			return;	// Scheda non collegata
		}
		sscanf (inbuf,"%04x  %02x %02x %02x %02x  %02x %02x %02x %02x  %02x %02x %02x %02x  %02x %02x %02x %02x", &j,
			vals, vals + 1, vals + 2, vals + 3, vals + 4, vals + 5, vals + 6, vals + 7,
			vals + 8, vals + 9, vals + 10, vals + 11, vals + 12, vals + 13, vals + 14, vals + 15);
		for (j = 0; j < 16; j++) {
			data [i - start + j] = vals [j];
		}
		masks [(i - start) / 8] = 0xff;
		masks [(i - start) / 8 + 1] = 0xff;
	}
	Serial_Container::s -> schPrompt ();
}

void XDataMemory::saveValue (int pos)
{
char buf [10];
	Serial_Container::s -> schPutc ('E');
	if (Serial_Container::s -> schWaitc (' ')) {
		return;
	}
	sprintf (buf,"%04X", pos + start);
	Serial_Container::s -> schPuts (buf);
	if (Serial_Container::s -> schWaitc ('-')) {
		return;
	}
	sprintf (buf,"%02X", data [pos]);
	Serial_Container::s -> schPuts (buf);
	if (Serial_Container::s -> schWaitc ('-')) {
		return;
	}
	Serial_Container::s -> schPutc ('\r');
	Serial_Container::s -> schPrompt ();
}



void XDataMemory::reload ()
{
int i;
	// Marca tutta la memoria come 'non letta'
	for (i = 0; i < size / 8; i++) {
		masks [i] = 0;
	}
	// E forza un ridisegno
	r -> redraw ();
}
